;+
; NAME:
;  astrom
; PURPOSE:
;  Astrometry from a digital image.
; DESCRIPTION:
;
; CATEGORY:
;  Astrometry
; CALLING SEQUENCE:
;  astrom,root,fileno
;
; INPUTS:
;  root   - Root of data file name (ie., 970309), must be a string.
;  fileno - File number to load (0-999), integer.
;
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD INPUT PARAMETERS:
;  BINFAC   - Binning factor for displayed image.  Default=2
;
;  EDIT     - Flag, if set allows interactive culling of bad astrometric
;                measurements in the reference net.
;
;  MAGLIM   - Limiting (faint) magnitude for catalog extraction (default=30.0)
;
;  NEWCAT   - Flag, if set forces the recreation of the STARFILE.
;
;  NOCLEAN  - Flag, if set suppresses the normal step of filtering out
;                cosmic ray strikes.  This steps doesn't take too long on
;                a Sun Ultra 1/170 but may be prohibitive on slower machines.
;
;  NOOBJECTS - Flag, if set suppresses the final step of measuring astrometric
;                unknowns.
;
;  OBJNAME  - Name of object to collect astrometry for.  By default, the
;                FITS header from the image is scanned for the OBJECT keyword
;                and this becomes the OBJNAME after compressing multiple blanks,
;                trimming leading and trailing blanks, and replacing single blanks
;                by "_".
;
;  OBJRAD   - Radius (in pixels) of object aperture for astrometry and
;                photometry.  Default=10.  The sky aperture is set between
;                objrad+10 and objrad+30.
;
;  PATH     - String, this is the name of the directory where the data are
;                stored.  The actual data directory used is PATH+'/'+root.
;                The default is '' (blank) and the file would be root.NNN
;                which would permit putting a leading path on the root.
;
;  PLASTFILE - This gives the name of the "plast" output file.  The default
;                is OBJNAME.pla.  This file contains a list of asteroids that
;                may be found on the image and is created by a separate
;                program.
;
;  PHOTSTARS - Flag.  If set, turns on a special mode that performs photometry
;                on all good astrometric reference stars.  The photometry is
;                added to the root+'.log' file.  Multiple reductions are
;                weeded out and the photometry log file is left sorted by
;                file.  The log file is in the ALTLOG format (see basphote).
;                The star name is automatically created from its coordinate.
;                Ex: a star at 12:12:34.2 and +04:23:45 would be named
;                NV1212342+042345.  The position used for the name comes from
;                the catalog, not astrometry from the image.
;
;  QUEUE     - String, name of printer to send output to.  If supplied, a
;                hardcopy of the image is generated along with a list of
;                stars identified.  If blank or not specified, no printed output
;                is generated.
;
;  RESFILE  - Filename where astrometric measurements are written to.  The
;                default file name is OBJNAME.ast.  Only one line per image
;                is allowed.  Subsequent measurements made by astrom will
;                override the measures for this image.
;
;  STARFILE - Filename where a list of astrometric reference stars is to
;                be found.  The default file name is OBJNAME.cat.  If this file
;                is not found, then this program will attempt to create the
;                file by calling "refnet", a program that accesses the
;                USNO A1.0 star catalog provided by David Monet of USNOFS.
;
;  WINDOW   - window number to display image into.
;
; OUTPUTS:
;  output is graphical and to a series of files.
;     astrom.inf - Saves 2-star astrometric solution from one invocation to next.
;     centers.dat - Records the image center for all measured frames.
;     root.log    - Photometry (if PHOTSTARS set)
;     objname.ast - Astrometry of object
;     objname.cat - Star catalog extraction
;     root.stars  - List of stars and coordinates for those where photometry
;                      was measured.  Intended for inclusion in starcat.dat
;                      (see GETSTARS.PRO or LOADSTAR.PRO).
;
; KEYWORD OUTPUT PARAMETERS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;  97/04/05, Written by Marc W. Buie, Lowell Observatory
;-
pro astrom,root,fileno, $
       WINDOW=window,EDIT=edit,OBJRAD=objrad,BINFAC=binfac, $
       STARFILE=starfile,RESFILE=resfile,PATH=in_path,OBJNAME=objname, $
       NOCLEAN=noclean,PLASTFILE=plastfile,NEWCAT=newcat,PHOTSTARS=photstars, $
       QUEUE=queue,NOOBJECTS=noobjects,MAXPHOTSIG=maxphotsig,MAGLIM=maglim, $
       KEYLIST=in_keylist

;help
   if badpar(root,7,0,CALLER='ASTROM: (root) ') then return
   if badpar(fileno,[2,3],0,CALLER='ASTROM: (fileno) ') then return
   
   if badpar(maglim,[0,2,3,4,5],0,CALLER='ASTROM: (MAGLIM) ',default=30.0) then return
   if badpar(binfac,[0,1,2,3],0,CALLER='ASTROM: (BINFAC) ',default=2) then return
   if badpar(edit,[0,1,2,3],0,CALLER='ASTROM: (EDIT) ',default=0) then return
   if badpar(newcat,[0,1,2,3],0,CALLER='ASTROM: (NEWCAT) ',default=0) then return
   if badpar(noclean,[0,1,2,3],0,CALLER='ASTROM: (NOCLEAN) ',default=0) then return
   if badpar(noobjects,[0,1,2,3],0,CALLER='ASTROM: (NOOBJECTS) ',default=0) then return
   if badpar(objname,[0,7],0,CALLER='ASTROM: (OBJNAME) ', $
                             default='[[DEFAULT]]') then return
   if badpar(objrad,[0,1,2,3,4,5],0,CALLER='ASTROM: (OBJRAD) ',default=10) then return
   if badpar(in_path,[0,7],0,CALLER='ASTROM: (PATH) ', $
                             default='[[DEFAULT]]') then return
   if badpar(maxphotsig,[0,2,3,4,5],0,CALLER='ASTROM: (MAXPHOTSIG) ',default=13000.0) then return
   if badpar(plastfile,[0,7],0,CALLER='ASTROM: (PLASTFILE) ', $
                              default='[[DEFAULT]]') then return
   if badpar(photstars,[0,1,2,3],0,CALLER='ASTROM: (PHOTSTARS) ',default=0) then return
   if badpar(queue,[0,7],0,CALLER='ASTROM: (QUEUE) ', $
                             default='') then return
   if badpar(resfile,[0,7],0,CALLER='ASTROM: (RESFILE) ', $
                             default='[[DEFAULT]]') then return
   if badpar(starfile,[0,7],0,CALLER='ASTROM: (STARFILE) ', $
                              default='[[DEFAULT]]') then return
   if badpar(window,[0,1,2,3],0,CALLER='ASTROM: (WINDOW) ',default=0) then return

   maglim=float(maglim) ; force type to float

   do_objects = not noobjects

   ;Initialize the default header correspondence list.
   hdrlist = [ [ 'AIRMASS',  'K', 'AIRMASS'    ], $
               [ 'DATE',     'K', 'DATE-OBS'   ], $
               [ 'DATETMPL', 'T', 'DD-MM-YYYY' ], $
               [ 'EXPDELTA', 'V', '0.0'        ], $
               [ 'EXPTIME',  'K', 'EXPTIME'    ], $
               [ 'FILTER',   'K', 'FILTERS'    ], $
               [ 'FILENAME', 'K', 'CCDFNAME'   ], $
               [ 'OBJECT',   'K', 'OBJECT'     ], $
               [ 'UT',       'K', 'UT'         ]  ]

   IF KEYWORD_SET( in_keylist ) THEN BEGIN
      IF exists( in_keylist ) THEN BEGIN
         readcol, in_keylist, c1, c2, c3, FORMAT='a,a,a', /SILENT
         c1 = STRUPCASE( c1 )
         c2 = STRUPCASE( c2 )
         c3 = STRUPCASE( c3 )
         FOR j=0, N_ELEMENTS(c1)-1 DO BEGIN
            w = WHERE( hdrlist(0,*) EQ c1(j), count )
            IF count NE 0 THEN BEGIN
               hdrlist(1:2,w)=[c2(j),c3(j)]
            ENDIF ELSE BEGIN
               hdrlist = [ [ hdrlist ], [ c1(j), c2(j), c3(j) ] ]
            ENDELSE
         ENDFOR
      ENDIF ELSE BEGIN
         MESSAGE, 'Error. Keylist file ' + in_keylist + ' not found.' + bel, /INFO
      ENDELSE
   ENDIF

   ; First, detangle the input information and get pointed at the data file
   ;  and directory.
   imfile=string(root,fileno,format='(a,".",i3.3)')
   IF in_path eq '[[DEFAULT]]' THEN BEGIN
      path = ''
   ENDIF ELSE BEGIN
      path = addslash(in_path)+root+'/'
   ENDELSE

   border=20

   thisfile=-1
   last=''
   IF exists('centers.dat') THEN BEGIN
      openr,lun,'centers.dat',/get_lun
      line=''
      WHILE not eof(lun) and thisfile lt fileno DO BEGIN
         readf,lun,line,format='(a)'
         words=str_sep(line,' ')
         fnf=str_sep(words(0),'.')
         thisfile = fix(fnf(1))
         IF thisfile eq fileno THEN BEGIN
            reads,words(1),hr,mn,sc,format='(i2,1x,i2,1x,f4.1)'
            cra=(float(hr)+float(mn)/60.0+sc/3600.0)*15.0/!radeg
            sign=''
            reads,words(2),sign,dg,mn,sc,format='(a1,i2,1x,i2,1x,i2)'
            if sign eq '-' then sign = -1 else sign = 1
            dmstorad,sign,dg,mn,sc,cdec
         ENDIF
         IF thisfile lt fileno THEN last=line
      ENDWHILE
      free_lun,lun

   ENDIF

   ; Read in image, and some header values
   IF not exists(path+imfile) THEN BEGIN
      message,'Image file: '+path+imfile+' not found.  Aborting.',/info
      return
   ENDIF
   raw=readfits(path+imfile,hdr,/silent)
   sz=size(raw)
   IF (sz(1)/binfac)*binfac ne sz(1) or (sz(2)/binfac)*binfac ne sz(2) THEN $
      raw=raw(0:(sz(1)/binfac)*binfac-1,0:(sz(2)/binfac)*binfac-1)
   IF objname eq '[[DEFAULT]]' THEN BEGIN
      objname=nobname(strtrim(strcompress(sxpar(hdr,'OBJECT')),2))
   ENDIF
   objname=strlowcase(objname)
   exptime=sxpar(hdr,'EXPTIME')
;   exptime=sxpar(hdr,'SHUTTER')
   hras=sxpar(hdr,'RA')
   hdecs=sxpar(hdr,'DEC')
   headercoord=0
   IF thisfile ne fileno THEN BEGIN
      reads,hras,hr,mn,sc,format='(i2,1x,i2,1x,f4.1)'
      hmstorad,hr,mn,sc,hra
      sign=''
      reads,hdecs,sign,dg,mn,sc,format='(a1,i2,1x,i2,1x,i2)'
      if sign eq '-' then sign = -1 else sign = 1
      dmstorad,sign,dg,mn,sc,hdec
      epoch=sxpar(hdr,'EPOCH')
      if epoch ne 2000 then begin
         hra = hra * !radeg
         hdec = hdec * !radeg
         precess,hra,hdec,epoch,2000.0
         hra = hra / !radeg
         hdec = hdec / !radeg
         rastr,hra,1,hras
         decstr,hdec,0,hdecs
         print,'Coordinates precessed to J2000 ',hras,' ',hdecs
      endif
      headercoord=1
   ENDIF
   uts=strtrim(sxpar(hdr,'UT'),2)
;   uts=sxpar(hdr,'STRTTIME')
   dates=sxpar(hdr,'DATE-OBS')
   print,dates,' [',uts,'] ',hras,' ',hdecs,' ',exptime
   uts=repchar(uts,':',' ')
   reads,dates,day,month,year,format='(i2,1x,i2,1x,i4)'
;   reads,dates,year,month,day,format='(i4,1x,i2,1x,i2)'
   IF year lt 100 THEN year = year+1900
   reads,uts,hour,min,sec,format='(i2,1x,i2,1x,i2)'
   hour = float(hour)+float(min)/60.0+float(sec)/3600.0 + exptime/2.0/3600.0
   jdcnv,year,month,day,hour,jd
   dayd = float(day)+hour/24.0

   IF starfile eq '[[DEFAULT]]' THEN BEGIN
      starfile = objname + '.cat'
   ENDIF

   IF plastfile eq '[[DEFAULT]]' THEN BEGIN
      plastfile = objname + '.pla'
   ENDIF

   ; See if PLAST file exists and set a flag for future use.
   doplast = exists(plastfile)

   ; Read in astrometry information saved from last run.  If not found,
   ;   ask for information.
   if exists('astrom.inf') then begin
      version=''
      openr,lun,'astrom.inf',/get_lun
      readf,lun,version,format='(a)'
      if version ne 'ASTROM v1.0' then begin
         print,'Illegal astrom.inf file, version tag is wrong.'
         return
      endif
      readf,lun,raoff,decoff
      readf,lun,pscale0
      readf,lun,rang0
      xflip=1
      yflip=1
      readf,lun,xflip,yflip,format='(i2,1x,i2)'
      free_lun,lun
   endif else begin
      raoff  = 0.0
      decoff = 0.0
      read,pscale0,prompt='Initial plate scale guess (arcsec/pixel)  '
      read,rang0,prompt='Rotation angle (degrees)  '
      ans=''
      read,ans,prompt='Is the X-axis flipped with respect to the sky? '
      if strmid(ans,0,1) eq 'y' then xflip=-1 else xflip=1
      read,ans,prompt='Is the Y-axis flipped with respect to the sky? '
      if strmid(ans,0,1) eq 'y' then yflip=-1 else yflip=1

      openw,lun,'astrom.inf',/get_lun
      printf,lun,'ASTROM v1.0'
      printf,lun,raoff,decoff
      printf,lun,pscale0
      printf,lun,rang0
      printf,lun,xflip,yflip,format='(i2,1x,i2)'
      free_lun,lun
   endelse

   goodcenter=0
   IF not exists(starfile) or newcat THEN BEGIN
      ; Attempt to get a star catalog file by running refnet
      ephem,jd,688,2+50,'e'+objname,eph
      objra=eph(0)
      objdec=eph(1)
      IF objra lt 0.0 THEN BEGIN
         read,h,m,s,prompt='RA of image center? '
         hmstorad,h,m,s,objra
         ans=''
         read,ans,prompt='Dec of image center? ',format='(a)'
         ans=strtrim(ans,2)
         if strmid(ans,0,1) eq '-' then sign=-1 else sign=1
         reads,ans,d,m,s
         dmstorad,sign,abs(d),m,s,objdec
      ENDIF
      goodcenter=1
      psize=fix(sqrt(sz(1)^2+sz(2)^2)*pscale0*1.05+0.5)
      refnet,objra,objdec,psize,psize,maglim,maglim,starfile
   ENDIF

   IF not exists(starfile) THEN BEGIN
      message,'The star catalog file '+starfile+' does not exist.  Aborting',/info
      return
   ENDIF

   IF not doplast and plastfile ne 'none' THEN BEGIN
      IF not goodcenter THEN BEGIN
         read,h,m,s,prompt='RA of image center? '
         hmstorad,h,m,s,objra
         ans=''
         read,ans,prompt='Dec of image center? ',format='(a)'
         ans=strtrim(ans,2)
         if strmid(ans,0,1) eq '-' then sign=-1 else sign=1
         reads,ans,d,m,s
         dmstorad,sign,abs(d),m,s,objdec
         psize=fix(sqrt(sz(1)^2+sz(2)^2)*pscale0*1.05+0.5)
      ENDIF
      rastr,objra,3,objras
      decstr,objdec,2,objdecs
      objras=repchar(objras,':',',')
      objdecs=repchar(objdecs,':',',')
      openw,luntmp,'plast.in',/get_lun
      printf,luntmp,'O'         ; output only
      printf,luntmp,'N'         ; selects J2000
      printf,luntmp,'0.1'       ; probability of being on image
      printf,luntmp,imfile      ; Title of plate
      printf,luntmp,year,month,dayd,format='(i4,",",i2.2,",",f5.2)'
      printf,luntmp,''          ; end of plate center input
      printf,luntmp,objras      ; plate center
      printf,luntmp,objdecs     ; plate center
      printf,luntmp,'1.0'       ; plate scale in arcsec/mm
      printf,luntmp,'r'         ; r - rectangular plate shape
      printf,luntmp,psize/25.4  ; width of plate in inches
      printf,luntmp,psize/25.4  ; height of plate in inches
      printf,luntmp,'n'         ; 
      printf,luntmp,'v'         ; magnitude to use (B/V)
      printf,luntmp,'30'        ; faint limit to grab
      printf,luntmp,'y'         ; 
      printf,luntmp,plastfile   ; output file name
      printf,luntmp,'688'       ; observatory code
      printf,luntmp,'Y'         ; 
      printf,luntmp,''          ; 
      printf,luntmp,'N'         ; 
      free_lun,luntmp
      spawn,'rm '+plastfile+' ; ' + $
            '/yoda/home/lhw/sac/xxx/plast < plast.in ; ' + $ ; >& /dev/null
            'rm plast.in plast.out'
      doplast = exists(plastfile)
   ENDIF

   IF headercoord THEN BEGIN
      cra=hra+raoff
      cdec=hdec+decoff
   ENDIF

   first=1

   info  = { $
      racen:  cra, $
      deccen: cdec, $
      pscale: pscale0, $
      rang:   rang0/!radeg, $
      xflip:  xflip, $
      yflip:  yflip, $
      xc:     sz(1)/2.0, $
      yc:     sz(2)/2.0 }

   ; Display image and setup coordinate system on display.
   dispim = rebin(raw,sz(1)/binfac,sz(2)/binfac,sample=0)
   setwin,window,xsize=sz(1)/binfac,ysize=sz(2)/binfac
   skysclim,dispim,lowval,hival,meanval,sigma
   lowval = meanval-7*sigma
   hival  = meanval+16*sigma
   savec=5
   bim = bytscl(dispim,min=lowval,max=hival,top=!d.n_colors-1-savec)+savec
   loadct,0,bottom=savec,/silent
   tvlct,r,g,b,/get
   r(0)=0
   g(0)=0
   b(0)=0
   r(1)=0
   g(1)=255
   b(1)=0
   r(2)=200
   g(2)=50
   b(2)=50
   r(3)=255
   g(3)=255
   b(3)=0
   r(4)=255
   g(4)=0
   b(4)=0
   tvlct,r,g,b
   tv,bim
   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
   oplot,[border,sz(1)-border,sz(1)-border,border,border], $
         [border,border,sz(2)-border,sz(2)-border,border],color=4

   ; Mark reminder spot
   if first and do_objects then begin
      print,'     Click a spot to remember'
      cursor,xloc,yloc,3
      if !err eq 4 then return
      first=0
   endif
   if do_objects then begin
      oplot,[-20,20,20,-20,-20]+xloc,[-20,-20,20,20,-20]+yloc,color=4
      empty ; flush graphics buffer output
   endif

   if noclean then begin
      image=raw
   endif else begin
      print,'       --> cleaning cosmic rays from image....'
      acre,raw,image,5,4
   endelse
   dispim = rebin(image,sz(1)/binfac,sz(2)/binfac,sample=0)
   bim = bytscl(dispim,min=lowval,max=hival,top=!d.n_colors-1-savec)+savec
   tv,bim
   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
   oplot,[border,sz(1)-border,sz(1)-border,border,border], $
         [border,border,sz(2)-border,sz(2)-border,border],color=4
   if do_objects then $
      oplot,[-20,20,20,-20,-20]+xloc,[-20,-20,20,20,-20]+yloc,color=4

   ; Read in list of stars.
; This is for files from Larry's gscplot
;   readcol,starfile,id,hr,m1,s1,dg,m2,s2,format='a,f,f,f,f,f,f',/silent
; This is for files from Bruce's web program
   readcol,starfile,hr,m1,s1,dgas,m2,s2,smag,format='d,d,d,a,d,d,f',/silent
   signas = strmid(dgas,0,1)
   dg = fix(strmid(dgas,1,2))
   sra=(float(hr)+float(m1)/60.0+s1/3600.0)*15.0/!radeg
   sign = replicate(1.0,n_elements(dg))
   z=where(signas eq '-',count)
   if count ne 0 then sign(z) = -1.0
   sdec=sign*(abs(dg)+m2/60.0+s2/3600.0)/!radeg
   ssz = (17-smag)/2 + 0.5
   z=where(ssz gt 4,count)
   if count ne 0 then ssz(z)=3.0
   z=where(ssz lt 0.5,count)
   if count ne 0 then ssz(z)=0.5
   bad=intarr(n_elements(sra))
   z=where(smag lt maglim,count)
   IF count eq 0 THEN BEGIN
      print,'None of the catalog stars are brighter than the limit,',maglim
      return
   ENDIF
   sra=sra(z)
   sdec=sdec(z)
   smag=smag(z)
   ssz=ssz(z)
   bad=bad(z)

   ; Read in the plast information (if found) and collect the information
   ;   for plotting
   IF doplast THEN BEGIN
      openr,lun,plastfile,/get_lun
      line=''
      FOR i=1,4 DO readf,lun,line,format='(a1)'
      numobj=0
      astname0=''
      WHILE not eof(lun) DO BEGIN
         readf,lun,astname0,mag0,format='(a23,f5.2)'
         readf,lun,line,format='(a1)'
         IF numobj eq 0 THEN BEGIN
            pastname=astname0
            pastmag=mag0
            numobj=1
         ENDIF ELSE BEGIN
            pastname=[pastname,astname0]
            pastmag=[pastmag,mag0]
            numobj=numobj+1
         ENDELSE
      ENDWHILE
      IF numobj eq 0 THEN doplast=0
      free_lun,lun
   ENDIF

   IF doplast THEN BEGIN
      astcode=strarr(numobj)
      print,'Asteroids found in plast support file that may be on image:'
      FOR i=0,numobj-1 DO BEGIN
         tmpstr=str_sep(strtrim(pastname(i),2),' ')
         astcode(i)='e'+tmpstr(0)
         print,i,astcode(i),pastname(i),pastmag(i), $
            format='(i3,1x,a9,1x,a23,1x,f4.1)'
      ENDFOR
      ephem,replicate(jd,numobj),688,23+50,astcode,eph
      astra=eph(0,*)
      astdec=eph(1,*)
      astvra=eph(18,*)
      astvdec=eph(19,*)
      asterr=eph(20,*)
      ephem,replicate(jd+1.0/24.0d0,numobj),688,2+50,astcode,eph
      astrap1=eph(0,*)
      astdecp1=eph(1,*)
;      FOR i=0,numobj-1 DO BEGIN
;         rastr,astra(i),1,str1
;         decstr,astdec(i),0,str2
;         rastr,astvra(i),1,str3
;         decstr,astvdec(i),0,str4
;         rastr,astrap1(i),1,str5
;         decstr,astdecp1(i),0,str6
;         print,i,pastname(i),str1,str2,asterr(i),str3,str4,str5,str6, $
;            format='(i3,1x,a15,1x,a,1x,a,1x,f4.1,2(2x,a,1x,a))'
;      ENDFOR
      astra=astra(*)
      astdec=astdec(*)
      astvra=astvra(*)
      astvdec=astvdec(*)
      asterr=asterr(*)
      astrap1=astrap1(*)
      astdecp1=astdecp1(*)
      c1=sqrt(astvra^2+astvdec^2)
      astvra=astvra/c1
      astvdec=astvdec/c1
      asterr=asterr/3600.0d0*!dpi/180.0d0
   ENDIF

retry:

   astrd2xy,sra,sdec,info,x,y
   z = where(x gt border and x lt sz(1)-border and $
             y gt border and y lt sz(2)-border,count)
   if count eq 0 and last ne '' then begin
      words=str_sep(last,' ')
      hr=0
      mn=0
      reads,words(1),hr,mn,sc,format='(i2,1x,i2,1x,f4.1)'
      hmstorad,hr,mn,sc,cra
      sign=''
      dg=0
      reads,words(2),sign,dg,mn,sc,format='(a1,i2,1x,i2,1x,i2)'
      if sign eq '-' then sign = -1 else sign = 1
      dmstorad,sign,dg,mn,sc,cdec
      last=''
      info.racen=cra
      info.deccen=cdec
      headercoord=0
      goto,retry
   endif else if count eq 0 then begin
      print,'No stars on frame.'
      ans=''
      read,ans,prompt='Do you want to (re)enter a corrected center? '
      if strmid(ans,0,1) eq 'n' then return
      read,h,m,s,prompt='RA of image center? '
      hmstorad,h,m,s,hra
      read,ans,prompt='Dec of image center? ',format='(a)'
      ans=strtrim(ans,2)
      if strmid(ans,0,1) eq '-' then sign=-1 else sign=1
      reads,ans,d,m,s
      dmstorad,sign,abs(d),m,s,hdec
;hmstorad,12,23,57.9,hra
;dmstorad,-1,4,38,3,hdec
      info.racen=hra+raoff
      info.deccen=hdec+decoff
      headercoord=0
      goto,retry
   endif
   for i=0,n_elements(x)-1 do $
      oplot,[x(i)],[y(i)],psym=4,color=2,symsize=ssz(i)

   ; Select first anchor star from catalog list
   print,'     Click on symbol for first anchor star.'
   cursor,xs1,ys1,3
   if !err eq 4 then return
   dist = sqrt((xs1-x)^2+(ys1-y)^2)
   zs1 = where(dist eq min(dist))
   zs1 = zs1(0)
   oplot,[x(zs1)],[y(zs1)],psym=4,color=4,symsize=2

   ; Click on the star in the image
   print,'     Click on the star in the image.'
   cursor,xs1,ys1,3
   if !err eq 4 then return
   gain=5.5
;   gain=1.0
   basphote,5.5,image,exptime,xs1,ys1,objrad,objrad+10,objrad+30,/nolog,/silent, $
      xcen=xs1m,ycen=ys1m
   oplot,[xs1m],[ys1m],psym=4,color=1

   ; Select seconds anchor star from catalog list
   print,'     Click on symbol for second anchor star.'
   cursor,xs2,ys2,3
   if !err eq 4 then return
   dist = sqrt((xs2-x)^2+(ys2-y)^2)
   zs2 = where(dist eq min(dist))
   zs2 = zs2(0)
   oplot,[x(zs2)],[y(zs2)],psym=4,color=4,symsize=2

   ; Click on the star in the image
   print,'     Click on the star in the image.'
   cursor,xs2,ys2,3
   if !err eq 4 then return
   basphote,5.5,image,exptime,xs2,ys2,objrad,objrad+10,objrad+30,/nolog,/silent, $
      xcen=xs2m,ycen=ys2m
   oplot,[xs2m],[ys2m],psym=4,color=1

   ; Separate in pixels
   pixsep = sqrt((xs2m-xs1m)^2+(ys2m-ys1m)^2)

   ; compute angular separation between reference stars
   angsep = acos(sin(sdec(zs1))*sin(sdec(zs2))+ $
                 cos(sdec(zs1))*cos(sdec(zs2))*cos(sra(zs2)-sra(zs1)))
   angsep = angsep * !radeg * 3600.0
   
   ; Compute plate scale
   pscale = angsep/pixsep
   info.pscale = pscale

   ; compute differential position of stars relative to their center.
   ra0 = mean([sra(zs1),sra(zs2)])
   dec0 = mean([sdec(zs1),sdec(zs2)])
   sx = info.xflip/pscale
   sy = info.yflip/pscale
   x0 = sx*(sra-ra0)*cos(dec0)*!radeg*3600.0
   y0 = sy*(sdec-dec0)*!radeg*3600.0

   ; Compute field rotation angle
   dra = x0(zs2)-x0(zs1)
   ddec= y0(zs2)-y0(zs1)
   dx  = xs2m-xs1m
   dy  = ys2m-ys1m
   alpha1=atan(ddec,dra)
   alpha2=atan(dy,dx)
   rang = alpha1 - alpha2
   info.rang = rang

   ; Redetermine the ra and dec center
   astxy2rd,(xs1m+xs2m)/2.0,(ys1m+ys2m)/2.0,info,ra,dec
   info.racen  = info.racen-ra+ra0
   info.deccen = info.deccen-dec+dec0

   ; Redetermine ra and dec offset for "good" header coordinates.
   IF headercoord THEN BEGIN
      raoff=info.racen-hra
      decoff=info.deccen-hdec
   ENDIF

   rastr,info.racen,1,ras
   decstr,info.deccen,0,decs
   print,'Image at',ras,decs,'scale',pscale,'arcsec/pixel', $
         rang*!radeg,' deg rotation', $
         format='(a,1x,a,1x,a,3x,a,f6.3,1x,a,2x,f7.2,1x,a)'
   cinfo=imfile+' '+ras+' '+decs
   repwrite,'centers.dat',imfile,cinfo

   tv,bim
   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
   oplot,[border,sz(1)-border,sz(1)-border,border,border], $
         [border,border,sz(2)-border,sz(2)-border,border],color=4
   if do_objects then $
      oplot,[-20,20,20,-20,-20]+xloc,[-20,-20,20,20,-20]+yloc,color=4
   astrd2xy,sra,sdec,info,x,y
   for i=0,n_elements(x)-1 do $
      oplot,[x(i)],[y(i)],psym=4,color=2,symsize=ssz(i)
   IF doplast THEN BEGIN
      astrd2xy,astra,astdec,info,astx,asty
      astrd2xy,astra+astvra*asterr,astdec+astvdec*asterr,info,astvxp,astvyp
      astrd2xy,astra-astvra*asterr,astdec-astvdec*asterr,info,astvxm,astvym
      astrd2xy,astrap1,astdecp1,info,astxp1,astyp1
      astxvar=[[astvxm],[astx],[astvxp]]
      astyvar=[[astvym],[asty],[astvyp]]
      oplot,astx,asty,psym=8,color=1,symsize=0.2
      FOR i=0,numobj-1 DO begin
print,i,asterr(i)*!radeg*3600.0,asterr(i)*!radeg*3600.0/pscale,pscale
         xyouts,astx(i),asty(i),' '+strtrim(string(i),2),color=1
         IF asterr(i)*!radeg*3600.0/pscale gt 5.0 THEN BEGIN
            oplot,astxvar(i,*),astyvar(i,*),color=1
            oplot,[astx(i),astxp1(i)],[asty(i),astyp1(i)],color=3
         ENDIF ELSE BEGIN
            oplot,[astx(i)],[asty(i)],psym=4,color=1,symsize=2.0
         ENDELSE
      ENDFOR
   ENDIF

   ; Select those stars on the frame
   nstars=n_elements(sra)
   xpos=fltarr(nstars)
   ypos=xpos
   perr=ypos+9999.0
   mag=perr
   err=perr
   fwhm=perr
   sig_max=fltarr(nstars)

   z = where(x le border or x ge sz(1)-border or $
             y le border or y ge sz(2)-border,count)
   IF count ne 0  THEN bad(z)=1

   z = where(x gt border and x lt sz(1)-border and $
             y gt border and y lt sz(2)-border and bad eq 0,count)
   if count ne 0 then begin
      for i=0,count-1 do begin
         basphote,5.5,image,exptime,x(z(i)),y(z(i)),objrad,objrad+10,objrad+30,/nolog,/silent, $
            xcen=xm,ycen=ym,fwhm=fwhm0,mag=mag0,err=err0,max=max0;,boxmrad=5
         xpos(z(i))=xm
         ypos(z(i))=ym
         perr(z(i))= sqrt((xm-x(z(i)))^2 + (ym-y(z(i)))^2)*pscale
         fwhm(z(i))=fwhm0
         mag(z(i))=mag0
         err(z(i))=err0
         sig_max(z(i))=max0
         rastr,sra(z(i)),1,ras
         decstr,sdec(z(i)),0,decs
;         print,x(z(i)),y(z(i)),ras,decs,x(z(i)),y(z(i)),xm,ym,perr(z(i)), $
;            format='(2(f6.1,1x),a,1x,a,1x,5(f6.1,1x))'
         setusym,-1
         oplot,[xpos(z(i))],[ypos(z(i))],psym=8,color=3,symsize=2.5
         setusym,1
         empty ; flush graphics buffer output
      endfor
      oplot,x(z),y(z),psym=4,color=2
   endif else begin
      print,'No stars on frame, unable to continue.'
      return
   endelse

   robomean,fwhm(z),3.0,0.5,meanfwhm,avgdev,stddev
   print,'Image size (FWHM) ',meanfwhm,' +/- ',stddev,meanfwhm*pscale, $
         format='(a,f4.1,a,f3.1," pixels  ",f4.1," arcsec")'
   for i=0,count-1 do begin
      IF perr(z(i)) gt 3.0 or abs(fwhm(z(i))-meanfwhm) ge 2.0*stddev THEN BEGIN
         oplot,[xpos(z(i))],[ypos(z(i))],psym=7,color=4,symsize=2.5
         bad(z(i))=1
      ENDIF
   endfor

   ; Convert from ra,dec to xi,eta (Smart, p283)
   astrd2sn,sra,sdec,info.racen,info.deccen,xi,eta
   xi  = xi*180.0d0/!dpi*3600.0d0
   eta = eta*180.0d0/!dpi*3600.0d0

   pass=1
   REPEAT BEGIN

      ; Now do full up plate solution on good positions
      zg = where(bad eq 0,countgood)
      oldbad=bad

      ; predefine variables for mysvdfit
      xfit=1
      efit=1
      var=1
      chisq=1
      sing=1

      ; Now setup independent variable vectors for x
      nterms=3
      xind=dblarr(nterms,countgood)
      xind(0,*) = 1.0
      xind(1,*) = xpos(zg)
      xind(2,*) = ypos(zg)
   ;   xind(3,*) = xpos(zg)*xpos(zg)
   ;   xind(4,*) = ypos(zg)*ypos(zg)
      weight  = replicate(1.0,countgood)

      ; Fit to xi coordinate
      cxi     = mysvdfit(xind,xi(zg),1,weight=weight, $
                         yfit=xfit,var=var,chisq=chisq,sing=sing)
      robomean,xi(zg)-xfit,3.0,0.5,avg,avgdev,stddev,vars,skew,kurt,nfinal
      cxisig   = sqrt(var)
      print,'xi:  chisq=',chisq/float(countgood-1), $
         ', scatter=',stddev, $
         ', worst resids',minmax(xi(zg)-xfit),countgood,pass, $ $
         format='(a,f6.2,a,f4.2,a,2(1x,f5.2),1x,i3," stars, pass ",i2)'
      print,cxi
      zbad=where(abs(xi(zg)-xfit) gt 3.0*stddev,countbad)
      IF countbad ne 0 THEN begin
         bad(zg(zbad))=1
         print,'   Setting ',countbad,' points to bad from xi fit', $
            format='(a,i3,a)'
      ENDIF
      setwin,10,xsize=300,ysize=300
      plot,xi(zg),xi(zg)-xfit,psym=7,symsize=0.5
;      IF edit THEN BEGIN
;         newbad=bad(zg)
;         markdata,xi(zg),xi(zg)-xfit,newbad, $
;            xtitle='xi  (arcsec)',ytitle='Residual (arcsec)'
;         bad(zg)=newbad
;      ENDIF

      ceta    = mysvdfit(xind,eta(zg),1,weight=weight, $
                         yfit=efit,var=var,chisq=chisq,sing=sing)
      robomean,eta(zg)-efit,3.0,0.5,avg,avgdev,stddev,vars,skew,kurt,nfinal
      cetasig = sqrt(var)
      print,'eta: chisq=',chisq/float(countgood-1), $
         ', scatter=',stddev, $
         ', worst resids',minmax(eta(zg)-efit),countgood,pass, $ $
         format='(a,f6.2,a,f4.2,a,2(1x,f5.2),1x,i3," stars, pass ",i2)'
      zbad=where(abs(eta(zg)-efit) gt 3.0*stddev,countbad)
      IF countbad ne 0 THEN BEGIN
         bad(zg(zbad))=1
         print,'   Setting ',countbad,' points to bad from xi fit'
      ENDIF
      print,ceta
      setwin,11,xsize=300,ysize=300
      plot,eta(zg),eta(zg)-efit,psym=7,symsize=0.5
      IF edit THEN BEGIN
         newbad=bad(zg)
         IF info.xflip eq 1 THEN xflip=0 ELSE xflip=1
         IF info.yflip eq 1 THEN yflip=0 ELSE yflip=1
         markdata,xi(zg),eta(zg),sqrt((xi(zg)-xfit)^2+(eta(zg)-efit)^2), $
            newbad,xtitle='xi (arcsec)',ytitle='eta (arcsec)', $
            xsize=600,ysize=600,xflip=xflip,yflip=yflip
         bad(zg)=newbad
;         newbad=bad(zg)
;         markdata,eta(zg),eta(zg)-efit,newbad, $
;            xtitle='eta (arcsec)',ytitle='Residual (arcsec)'
;         bad(zg)=newbad
      ENDIF

      zchg = where(bad ne oldbad, countchange)
      IF countchange ne 0 THEN BEGIN
         setwin,window
         tv,bim
         plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
            xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
         oplot,[border,sz(1)-border,sz(1)-border,border,border], $
               [border,border,sz(2)-border,sz(2)-border,border],color=4
         if do_objects then $
            oplot,[-20,20,20,-20,-20]+xloc,[-20,-20,20,20,-20]+yloc,color=4
         for i=0,nstars-1 do begin
            if bad(i) eq 1 then $
               oplot,[xpos(i)],[ypos(i)],psym=7,color=4,symsize=2.5 $
            else begin
               setusym,-1
               oplot,[xpos(i)],[ypos(i)],psym=8,color=3,symsize=2.5
               setusym,1
            endelse
         endfor
      ENDIF

      pass=pass+1

   ENDREP UNTIL countchange eq 0

   zok=where((sig_max le maxphotsig) and (bad eq 0) and (err lt 0.15),countok)
   IF countok gt 0 THEN BEGIN
      robomean,smag(zok)-mag(zok),3.0,0.5,magdiff,avgdev,stddev
      zok1=where(sig_max le maxphotsig and bad eq 0 and $
                abs(smag-mag-magdiff) lt 2.0*stddev,countok1)
      meanerr,smag(zok1)-mag(zok1),err(zok1),magdiff,avgdev,stddev
      print,'Mean catalog magnitude difference ',magdiff,stddev/sqrt(countok1-1), $
            'from',countok1,'stars',format='(a,f7.3," +/- ",f6.3,1x,a,1x,i3,1x,a)'
      fwhmrange=minmax(fwhm(zok))
      zok=where((sig_max le maxphotsig) and (err lt 0.15) and $
                fwhm ge fwhmrange(0) and fwhm le fwhmrange(1),countok)
   ENDIF ELSE BEGIN
      magdiff = 0.0
      print,'Warning: no stars in valid photometric range, magdiff set to 0.'
      print,'Range in peak counts: ',minmax(sig_max(where(bad eq 0)))
   ENDELSE

   ; Recompute the image center and update value in 'centers.dat'
   xi =(cxi(0) +info.xc*cxi(1) +info.yc*cxi(2))/3600.0d0*!dpi/180.0d0
   eta=(ceta(0)+info.xc*ceta(1)+info.yc*ceta(2))/3600.0d0*!dpi/180.0d0
   astsn2rd,xi,eta,info.racen,info.deccen,ra,dec
   rastr,ra,3,ras
   decstr,dec,2,decs
   print,'final center ',ras,' ',decs
   cinfo=imfile+' '+ras+' '+decs
   repwrite,'centers.dat',imfile,cinfo

   ; Save 2-star solution for next run.
   openw,lun,'astrom.inf',/get_lun
   printf,lun,'ASTROM v1.0'
   printf,lun,raoff,decoff
   printf,lun,info.pscale
   printf,lun,info.rang*!radeg
   printf,lun,info.xflip,info.yflip,format='(i2,1x,i2)'
   free_lun,lun

   IF photstars and countok gt 0 THEN BEGIN
      setwin,window
      tv,bim
      plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
         xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
      oplot,[border,sz(1)-border,sz(1)-border,border,border], $
            [border,border,sz(2)-border,sz(2)-border,border],color=4
      setusym,-1
      FOR i=0,countok-1 DO BEGIN
         oplot,[xpos(zok(i))],[ypos(zok(i))],psym=8,color=3,symsize=2.5
         xyouts,xpos(zok(i)),ypos(zok(i)),'  '+strtrim(string(i),2),color=3
      ENDFOR
      setusym,1

      hardim,bytscl(image,min=lowval,max=hival,top=255),0,255, $
         title=imfile+' '+objname, $
         autosize=2,queue=queue,/negative,width=14.0,/noprint
      setusym,-1
      FOR i=0,countok-1 DO BEGIN
         oplot,[xpos(zok(i))],[ypos(zok(i))],psym=8,symsize=2.5
         xyouts,xpos(zok(i)),ypos(zok(i)),'  '+strtrim(string(i),2),/data
      ENDFOR
      setusym,1
      device,/close
      display

      hardcopy = queue ne ''
      IF hardcopy THEN BEGIN
         spawn,'lpr -P'+queue+' idl.ps'
         print,'output sent to printer office'
      ENDIF

      logfile=root+'.log'
      starfile=root+'.stars'
      openw,lunstar,starfile,/append,/get_lun
      IF hardcopy THEN BEGIN
         openw,lunlist,'tmp.xxx',/get_lun
         printf,lunlist,'Image: ',imfile,'  object: ',objname
         printf,lunlist,' '
      ENDIF
      print,'Doing automatic photometry on the good stars to ',logfile
      ; Construct object names
      starname=strarr(countok)
      FOR i=0,countok-1 DO BEGIN
         rastr,sra(zok(i)),1,srastr
         decstr,sdec(zok(i)),0,sdecstr
         starname(i)='NV'+strmid(srastr,0,2)+strmid(srastr,3,2)+ $
                          strmid(srastr,6,2)+strmid(srastr,9,1)+ $
                          strmid(sdecstr,0,3)+strmid(sdecstr,4,2)+ $
                          strmid(sdecstr,7,2)
         printf,lunstar,starname(i),' ',srastr,' ',sdecstr,'  0.000  0.000 '
         IF hardcopy THEN $
            printf,lunlist,i,' ',starname(i),' ',srastr,' ',sdecstr
      ENDFOR
      free_lun,lunstar
      IF hardcopy THEN BEGIN
         free_lun,lunlist
         spawn,'lpr -P'+queue+' tmp.xxx'
      ENDIF
      spawn,'sort '+starfile+' | uniq > tmp.xxx; mv tmp.xxx '+starfile
      filter=strtrim(sxpar(hdr,'FILTERS'),2)
;      filter=strtrim(sxpar(hdr,'FILTER'),2)
;      filter=strtrim(sxpar(hdr,'FILTERL'),2)
      if exists(logfile) then $
         logmanip,logfile,/JUSTCLEAN,DELETEFILE=imfile
      objnum=10
      basphote,5.5,image,exptime,xpos(zok),ypos(zok), $
               objrad,objrad+10,objrad+30,logfile,objnum,/ALTLOG,/SILENT, $
               FNAME=imfile,FILTER=filter,JD=jd,NAME=starname,PSCALE=info.pscale
      logmanip,logfile,/JUSTCLEAN
   ENDIF ; photstars block

   objnum=0
   IF do_objects THEN BEGIN
      theta=findgen(361.0)/!radeg
      xcirc=objrad*cos(theta)
      ycirc=objrad*sin(theta)
      ; Get ready for astrometry on unknown
      doneone=0
      REPEAT BEGIN
         setwin,window
         tv,bim
         plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
            xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
         oplot,[border,sz(1)-border,sz(1)-border,border,border], $
               [border,border,sz(2)-border,sz(2)-border,border],color=4
         oplot,[-20,20,20,-20,-20]+xloc,[-20,-20,20,20,-20]+yloc,color=4
         IF doplast THEN BEGIN
            oplot,astx,asty,psym=8,color=1,symsize=0.2
            FOR i=0,numobj-1 DO BEGIN
               xyouts,astx(i),asty(i),' '+strtrim(string(i),2),color=1
               IF asterr(i)*!radeg*3600.0/pscale gt 5.0 THEN BEGIN
                  oplot,astxvar(i,*),astyvar(i,*),color=1
                  oplot,[astx(i),astxp1(i)],[asty(i),astyp1(i)],color=3
               ENDIF ELSE BEGIN
                  oplot,[astx(i)],[asty(i)],psym=4,color=1,symsize=2.0
               ENDELSE
            ENDFOR
         ENDIF

         setusym,-1
         FOR i=0,nstars-1 DO BEGIN
            IF bad(i) eq 0 THEN BEGIN
               oplot,[xpos(i)],[ypos(i)],psym=8,color=3,symsize=2.5
            ENDIF ELSE BEGIN
               oplot,[xpos(i)],[ypos(i)],psym=8,color=2,symsize=2.5
            ENDELSE
         ENDFOR
         if doneone then oplot,[xm],[ym],psym=4,color=4
         setusym,1
         print,'Left click '+objname+' to measure (middle-new object, right-exit)'
         cursor,x,y,3
         button = !err
         IF button eq 1 THEN BEGIN
            basphote,5.5,image,exptime,x,y,objrad,objrad+10,objrad+30,/nolog, $
               /silent,xcen=xm,ycen=ym,mag=mag0,fwhm=fwhm0;,boxmrad=5
            astmag=mag0+magdiff
            xm0=fix(xm+0.5)
            ym0=fix(ym+0.5)
            dw=fix(ceil(meanfwhm*2.0))
            IF fwhm0 gt 1.5 THEN BEGIN
               radp,image(xm0-dw:xm0+dw,ym0-dw:ym0+dw),xm-(xm0-dw),ym-(ym0-dw), $
                    r,i,rfwhm,rcoefs,rfit,fzwid=fwhm0
            ENDIF ELSE BEGIN
               radp,image(xm0-dw:xm0+dw,ym0-dw:ym0+dw),xm-(xm0-dw),ym-(ym0-dw), $
                    r,i,rfwhm,rcoefs,rfit,fzwid=meanfwhm
            ENDELSE
            idx=sort(r)
            setwin,12
            plot,r(idx),i(idx),psym=4
            oplot,r(idx),rfit(idx),color=3
            xyouts,0.6,0.8,'FWHM='+ $
                           strcompress(string(fwhm0,format='(f6.1)'),/remove)+ $
                           ' pixels',/normal
            xyouts,0.6,0.75,'FWHM='+ $
                           strcompress(string(fwhm0*pscale,format='(f6.1)'),/remove)+ $
                           ' arcsec',/normal
            zbin=6
            zsz=zbin*(2*dw+1)
            setwin,13,xsize=zsz,ysize=zsz,/show
            tv,rebin( $
                  bytscl( $
                     image(xm0-dw:xm0+dw,ym0-dw:ym0+dw), $
                           min=lowval,max=hival,top=!d.n_colors-1-savec)+savec, $
                  zsz,zsz,/sample)
            plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
               xr=[xm0-dw-0.5,xm0+dw+0.5],yr=[ym0-dw-0.5,ym0+dw+0.5], $
               xstyle=5,ystyle=5,/noerase
            oplot,xcirc+xm,ycirc+ym,color=1

            xi =(cxi(0)+xm*cxi(1)+ym*cxi(2))/3600.0d0*!dpi/180.0d0
            eta=(ceta(0)+xm*ceta(1)+ym*ceta(2))/3600.0d0*!dpi/180.0d0
            astsn2rd,xi,eta,info.racen,info.deccen,ra,dec
            rastr,ra,3,ras
            decstr,dec,2,decs
            infostr=string(imfile,jd,ras,decs,astmag, $
                           format='(a,1x,f13.5,1x,a,1x,a,1x,f4.1)')
            print,infostr,' FWHM ',fwhm0*info.pscale,' arcsec', $
                  format='(a,a,f4.1,a)'

            doneone=1

         ENDIF ELSE BEGIN ; new object or quit

            ; Save data if something was measured.
            IF doneone THEN BEGIN
               IF resfile eq '[[DEFAULT]]' THEN BEGIN
                  resfile = objname + '.ast'
               ENDIF
               repwrite,resfile,imfile,infostr
               resfile='[[DEFAULT]]'
               doneone=0
               IF photstars THEN BEGIN
                  basphote,5.5,image,exptime,xm,ym,objrad,objrad+10,objrad+30, $
                           logfile,objnum,/ALTLOG,/SILENT,FNAME=imfile, $
                           FILTER=filter,JD=jd,NAME=objname,PSCALE=info.pscale
               ENDIF
            ENDIF

            ; Ask for new name if middle click.
            IF button eq 2 THEN BEGIN
               read,objname,prompt='New object name? '
               objname=nobname(strtrim(strcompress(objname),2))
            ENDIF

         ENDELSE
      ENDREP UNTIL button eq 4 ; loop on measuring object

   ENDIF  ; do_objects block

end

