;+
; NAME: 
;  clscan
; PURPOSE:
;  Scan a group of raw OSIRIS XD frames and find rough spectral location.
; DESCRIPTION:
; CATEGORY:
;  Spectroscopy
; CALLING SEQUENCE:
;  clscan,calib,root,start,last,frno,mate,RANGE=range,PATH=path
; INPUTS:
;  calib- Anonymous structure containing all pertinent calibration
;           information.  This structure is usually loaded beforehand using
;           the routine, "ldcalir"
;  root - string containing the root of the file name (with leading path
;         if desired).  DO NOT include the . between the root and suffix.
;  start  - First spectrum number of sequence to scan.
;  last   - Last spectrum number of sequence to scan, if negative, this number
;             is taken to be the number of spectra to scan.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  PATH - optional string that points to the directory containing the data.
;           This information is not used if the root already begins with '/'.
;           If root is not an absolute pathname, then PATH is prepended to
;           root for READ operations.  This path is not used for saving.
;           This allows reading from one directory (possible a read only area)
;           and then saving to the current directory.
;  RANGE - range of column numbers to average in extracted spectral slice to
;           find rough location of spectrum.  Default is entire spectrum.
;
; OUTPUTS:
;  frno - A vector of frame numbers.
;  mate - A vector that gives a suggested frame number from this set for
;           a sky pair match that does not overlap the object.
;  
;  Also, information on location for each spectrum is printed to the console.
;
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
;  Specifically written for OSIRIS cross-dispersed spectral data.
; PROCEDURE:
; MODIFICATION HISTORY:
;  95/09/27, Written by Marc W. Buie, Lowell Observatory
;  96/05/29, MWB, changed 4th argument from NSPEC to LAST.
;-
pro clscan,calib,root,start,last,frno,mate, $
   PATH=path,RANGE=range,EXCLUDE=exclude

if badpar(calib,8,1,CALLER='clscan (calib) ') then return
if badpar(root,7,0,CALLER='clscan (root) ') then return
if badpar(start,[2,3],0,CALLER='clscan (start) ') then return
if badpar(last,[2,3],0,CALLER='clscan (last) ') then return
if badpar(range,[0,2,3],1,CALLER='clscan (range) ',DEFAULT=[0,calib.npts-1]) then return
if badpar(exclude,[0,2,3],[0,1],CALLER='clscan (exclude) ',DEFAULT=-1) then return
if badpar(path,[0,7],0,caller='clscan: (path) ',default='./') then return

if strmid(root,0,1) eq '/' then begin
   newroot = root
endif else begin
   newroot = addslash(path)+root
endelse

if last lt 0 then $
   nspec=abs(last) $
else $
   nspec =  last - start + 1

frno=indgen(nspec)+start
for i=0,nspec-1 do begin
   z=where(frno(i) eq exclude,count)
   if count ne 0 then frno(i)=-1
endfor
z=where(frno ne -1,count)
if count eq 0 then $
   message,'Error ** you have excluded all frames, nothing to do.'

frno = frno(z)
nsp = n_elements(frno)

path=addslash(path)

loc = intarr(nsp)
perloc = fltarr(nsp)

ranper = 1.0/float(calib.height)
for i=0,nsp-1 do begin
   fname = root+'.'+string(frno(i),format='(i3.3)')
   getstrip,calib,path+root,frno(i),strip
   maxloc,strip,xpos,ypos,range(0),range(1),/x
   hist=histogram(ypos)
   hloc = where(hist eq max(hist))
   loc(i) = hloc(0)
   perloc(i) = float(hist(loc(i)))/float(n_elements(ypos))
;   if perloc(i) lt 2.0*ranper then begin
;      sz=size(strip)
;      for j=0,sz(1)-1 do begin
;         med = median(strip(j,*))
;         strip(j,*) = strip(j,*)-med
;      endfor
;      asdf
;   endif
endfor

minsep=8
mate = intarr(nsp)
matedis = intarr(nsp)

; Rule one, find nearest frame number spectrum that is further than MINSEP
;   away from current spectrum.
for i=0,nsp-1 do begin
   fname = root+'.'+string(frno(i),format='(i3.3)')
   ; distance in frame number from current frame
   fdist = abs(frno - frno(i))
   ; distance in pixels from current frame location.
   ydist = abs(loc - loc(i))

   ; find those that are further than MINSEP
   z = where(ydist ge minsep,count)

   ; there are frames futher than minsep, get the closest in frame distance.
   if count ne 0 then begin
      ydistz = ydist(z)
      fdistz = fdist(z)
      specz  = frno(z)
      z = where(fdistz eq min(fdistz))
      mate(i) = specz(z(0))
      matedis(i) = ydistz(z(0))

   ; nothing is further than minsep, take the furthest but still closest in frame #.
   endif else begin
      z = where(ydist eq max(ydist))
      ydistz = ydist(z)
      fdistz = fdist(z)
      specz  = frno(z)
      z = where(fdistz eq min(fdistz))
      mate(i) = specz(z(0))
      matedis(i) = ydistz(z(0))
   endelse
   if perloc(i) lt 2.0*ranper then $
      comment = 'very faint check location' $
   else $
      comment =''
   print,fname,loc(i),mate(i),matedis(i),comment

endfor

end

