;+
; NAME:
;  fieldobs
; PURPOSE:
;  Real-time display of standard star locations in the sky
; DESCRIPTION:
;
; CATEGORY:
;  Astronomy
; CALLING SEQUENCE:
;  fieldobs,file,AMCRIT=amcrit,OBSFILE=obsfile,OBSCODE=obs
; INPUTS:
;  file - List of objects (for file format see LOADSTARS) to display
;
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD INPUT PARAMETERS:
;  AMCRIT  - Maximum airmass for plotted display (default = 3.0)
;  OBSFILE - Location of observatory information file, default is
;              /pub/sac0/elgb/data/obscode.dat
;  OBS     - Observatory code, default = 688, Lowell Observatory
;
; OUTPUTS:
;
; KEYWORD OUTPUT PARAMETERS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;  97/05/28, Written by Marc W. Buie, Lowell Observatory
;-
pro fieldobs,file,AMCRIT=amcrit,OBSFILE=obsfile,OBSCODE=obs

   if badpar(amcrit,[0,2,3,4,5],0,caller='FIELDOBS: (amcrit) ',default=3.0) then return
   if badpar(obsfile,[0,7],0,CALLER='FIELDOBS: (OBSFILE) ', $
                  DEFAULT='/pub/sac0/elgb/data/obscode.dat') then return
   if badpar(obs,[0,2,3],0,caller='FIELDOBS: (obscode) ',default=688) then return

   obs=688
   rdobscod,code,alllon,rhosinp,rhocosp,obsname,FILE=obsfile
   idx=where(obs eq code,count)
   idx=idx(0)
   if (count eq 1) then begin
      lon = (360.0-alllon(idx))/180.0*!pi
      lat = atan(rhocosp(idx),rhosinp(idx))
      name=strtrim(obsname(idx),2)
   endif else begin
      obs = -1
   endelse

   ; Read the special positions file, these won't be in the star catalog
   openr,luns,file,/get_lun
   sname=''
   sinfo=''
   sign=''
   num=0
   while not eof(luns) do begin
      readf,luns,sname,rh,rm,rs,sign,dd,dm,ds,ep,sinfo, $
         format='(a16,2(1x,i2),1x,f4.1,1x,a1,3(i2,1x),f6.1,1x,a)'
      newra   = rh + rm/60.0 + rs/3600.0
      newdec  = dd + dm/60.0 + ds/3600.0
      if sign eq '-' then newdec = -1.0*newdec
      if num eq 0 then begin
         id      = sname
         ra      = newra
         dec     = newdec
         equi    = ep
         rapm    = 0.0
         decpm   = 0.0
         epoch   = ep
         comment = sinfo
      endif else begin
         id      = [id,     sname]
         ra      = [ra,     newra]
         dec     = [dec,    newdec]
         equi    = [equi,   ep]
         rapm    = [rapm,   0.0]
         decpm   = [decpm,  0.0]
         epoch   = [epoch,  ep]
         comment = [comment,sinfo]
      endelse
      num=num+1
   endwhile
   free_lun,luns

   ra = ra*15.0/!radeg
   dec = dec/!radeg

   crital = 0.5*!pi - acos(1.0/amcrit)

   while 1 do begin

      curtime = systime(1) / 86400.0d0
      curjd   = curtime + 2440587.5d0
      dt = 15.0/60.0/24.0

      jd=[curjd,curjd+dt,curjd+dt*2]

      moonpos,curjd,moonra,moondec
      sunpos,curjd,sunra,sundec
      mam = airmass(curjd,moonra,moondec,lat,lon,lha=mlha,alt=malt,/hardie)

      jdstr,curjd,0,curtimstr
      am=fltarr(3,num)
      lha=fltarr(3,num)
      for i=0,num-1 do begin
         am(*,i) = airmass(jd,ra(i),dec(i),lat,lon,lha=lha0,/hardie)
         lha(*,i) = lha0
      endfor
      z=where(am lt amcrit)
      lha = lha*!radeg/15.0
      mlha = mlha*!radeg/15.0

      print,' '
      print,curtimstr
      print,'       moon altitude ',malt*!radeg,format='(a,f5.1," degrees")'
      plot,[0],[1],/nodata,xr=minmax(lha(z))+[-0.1,0.2],yr=[amcrit,1.0], $
         xtitle='W      Hour Angle       E',ytitle='Airmass',title=curtimstr
      setusym,-1
      oplot,[mlha],[mam],psym=8,symsize=2.0
      setusym,1
      if malt gt 0. then $
         xyouts,0.5,0.2,'Moon is up',align=0.5,/normal
      for i=0,num-1 do begin
         if min(am(*,i)) lt amcrit then begin
            print,i,' ',id(i),' ',lha(0,i),am(0,i)
            oplot,lha(*,i),am(*,i),psym=-4
            xyouts,max(lha(*,i)),min(am(*,i)),' '+strtrim(string(i),2)
         endif
      endfor
      empty ; flush graphics buffer

      spawn,'sleep 300'

   endwhile

end
