;+
; NAME:
;  findobj
; PURPOSE:
;  Locate image changes with 3-plane color overlays
; DESCRIPTION:
;
; CATEGORY:
;  Astronomy
; CALLING SEQUENCE:
;  findobj,root,image1,image2,image3,NOCLEAN=noclean
;
; INPUTS:
;  root - String containing the root of the file name (no path).
;  image1 - Image number (integer) to load into the red image plane.
;  image2 - Image number to load into the green image plane.
;  image3 - Image number to load into the blue image plane (default=image2)
;
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD INPUT PARAMETERS:
;  NOCLEAN - Flag, if set will suppress the automatic removal of cosmic rays.
;              Note: this step can take a very long time if you don't have
;              a fast machine.  On an Ultra 1/170 I find I will occasionally
;              supress cleaning.
;
; OUTPUTS:
;  No explicit outputs but you are prompted for a final hardcopy in either
;    color or black and white.
;
; KEYWORD OUTPUT PARAMETERS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;  You must have a 24-bit color display to use this program.
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;  97/04/17 - Written by Marc W. Buie, Lowell Observatory
;-
pro findobj,root,image1,image2,image3,NOCLEAN=noclean

   IF badpar(image3,[0,2,3],0,caller='FINDOBJ: (image3) ',default=image2) THEN return
   if badpar(noclean,[0,1,2,3],0,CALLER='FINDOBJ: (NOCLEAN) ',default=0) then return

   fmt='(a,".",i3.3)'
   imfile1 = string(root,image1,format=fmt)
   imfile2 = string(root,image2,format=fmt)
   imfile3 = string(root,image3,format=fmt)

   ; Read in image, and some header values
   raw=readfits(imfile1,hdr1,/silent)
   if noclean then im1=raw else acre,raw,im1,5,4
   raw=readfits(imfile2,hdr2,/silent)
   if noclean then im2=raw else acre,raw,im2,5,4
   IF image3 ne image2 THEN BEGIN
      raw=readfits(imfile3,hdr3,/silent)
      if noclean then im3=raw else acre,raw,im3,5,4
   ENDIF ELSE BEGIN
      im3=im2
   ENDELSE
   sz=size(im1)
   nx = sz(1)
   ny = sz(2)

   ; Display image and setup coordinate system on display.
   binfac = 2
   dim1 = rebin(im1,nx/binfac,ny/binfac,sample=0)
   dim2 = rebin(im2,nx/binfac,ny/binfac,sample=0)
   dim3 = rebin(im3,nx/binfac,ny/binfac,sample=0)

   skysclim,dim1,low1,hi1,mean1,sigma1
   low1 = mean1-7*sigma1
   hi1  = mean1+16*sigma1
   skysclim,dim2,low2,hi2,mean2,sigma2
   low2 = mean2-7*sigma2
   hi2  = mean2+16*sigma2
   skysclim,dim3,low3,hi3,mean3,sigma3
   low3 = mean3-7*sigma3
   hi3  = mean3+16*sigma3

   setwin,0,xsize=nx/binfac,ysize=ny/binfac,/show
   bim1 = bytscl(dim1,min=low1,max=hi1,top=255)
   tv,bim1
   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase

   print,'Click on a reference star'
   cursor,xloc1,yloc1,3
   if !err eq 4 then return
   basphote,5.5,im1,1.0,xloc1,yloc1,10,20,40,/nolog,/silent, $
      xcen=x1m,ycen=y1m,flux=fl1
   oplot,[x1m],[y1m],psym=4,color='ff0000'xl
   x1m = fix(x1m+0.5)
   y1m = fix(y1m+0.5)

   setwin,1,xsize=nx/binfac,ysize=ny/binfac,/show
   bim2 = bytscl(dim2,min=low2,max=hi2,top=255)
   tv,bim2

   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
   oplot,[-20,20,20,-20,-20]+x1m,[-20,-20,20,20,-20]+y1m,color='ff2000'xl
   if image2 ne image1 then begin
      cursor,xloc2,yloc2,3
      if !err eq 4 then return
      basphote,5.5,im2,1.0,xloc2,yloc2,10,20,40,/nolog,/silent, $
         xcen=x2m,ycen=y2m,flux=fl2
      x2m = fix(x2m+0.5)
      y2m = fix(y2m+0.5)
   endif else begin
      xloc2=xloc1
      yloc2=yloc1
      x2m = x1m
      y2m = y1m
      fl2 = fl1
   endelse
   oplot,[x2m],[y2m],psym=4,color='ff0000'xl

   setwin,2,xsize=nx/binfac,ysize=ny/binfac,/show
   bim3 = bytscl(dim3,min=low3,max=hi3,top=255)
   tv,bim3

   plot,[0],[1],/nodata,xmargin=[0,0],ymargin=[0,0], $
      xr=[0,sz(1)-1],yr=[0,sz(2)-1],xstyle=5,ystyle=5,/noerase
   oplot,[-20,20,20,-20,-20]+x1m,[-20,-20,20,20,-20]+y1m,color='ff2000'xl
   if image3 ne image2 then begin
      cursor,xloc3,yloc3,3
      if !err eq 4 then return
      basphote,5.5,im3,1.0,xloc3,yloc3,10,20,40,/nolog,/silent, $
         xcen=x3m,ycen=y3m,flux=fl3
      x3m = fix(x3m+0.5)
      y3m = fix(y3m+0.5)
   endif else begin
      xloc3=xloc2
      yloc3=yloc2
      x3m = x2m
      y3m = y2m
      fl3 = fl2
   endelse
   oplot,[x3m],[y3m],psym=4,color='ff0000'xl

   dleft  = min([x1m,x2m,x3m])
   dright = min(nx-[x1m,x2m,x3m])-1
   dup    = min(ny-[y1m,y2m,y3m])-1
   ddown  = min([y1m,y2m,y3m])

   print,dleft,dright,dup,ddown
   print,'Image 1 section ',x1m-dleft,x1m+dright,y1m-ddown,y1m+dup
   print,'Image 2 section ',x2m-dleft,x2m+dright,y2m-ddown,y2m+dup
   print,'Image 3 section ',x3m-dleft,x3m+dright,y3m-ddown,y3m+dup
   print,'fluxes ',fl1,fl2,fl3,fl2/fl1,fl3/fl1
   print,'means  ',mean1,mean2,mean3
   print,'sigmas ',sigma1,sigma2,sigma3
   sig=(sigma1+sigma2+sigma3)/3.0
   sig1 = sig
   sig2 = sig*fl2/fl1
   sig3 = sig*fl3/fl1

   xsz=dright+dleft+1
   ysz=dup+ddown+1

   setwin,3,xsize=xsz,ysize=ysz,/show
   fim1 = bytscl(im1,min=mean1-7*sig1,max=mean1+16*sig1,top=255)
   fim2 = bytscl(im2,min=mean2-7*sig2,max=mean2+16*sig2,top=255)
   fim3 = bytscl(im3,min=mean3-7*sig3,max=mean3+16*sig3,top=255)
   tv,[[[fim1(x1m-dleft:x1m+dright,y1m-ddown:y1m+dup)]], $
       [[fim2(x2m-dleft:x2m+dright,y2m-ddown:y2m+dup)]], $
       [[fim3(x3m-dleft:x3m+dright,y3m-ddown:y3m+dup)]]],true=3

   ans=''
   read,ans,prompt='Hardcopy? (c/b/n)'
   IF strmid(ans,0,1) eq 'c' THEN BEGIN
      npix = max([xsz,ysz])
      IF npix lt 600 THEN $
         scale = 25.0 $
      ELSE IF npix lt 1064 THEN $
         scale = 56.0 $
      ELSE IF npix lt 2100 THEN $
         scale = 112.0 $
      ELSE $
         scale = float(npix)/15.0
      hardim,[[[fim1(x1m-dleft:x1m+dright,y1m-ddown:y1m+dup)]], $
             [[fim2(x2m-dleft:x2m+dright,y2m-ddown:y2m+dup)]], $
             [[fim3(x3m-dleft:x3m+dright,y3m-ddown:y3m+dup)]]],0,255, $
             autosize=1,queue='chani',true_color=3,width=npix/scale, $
             title='R:'+imfile1+' G:'+imfile2+' B:'+imfile3
   ENDIF ELSE IF strmid(ans,0,1) eq 'b' THEN BEGIN
      npix = max([xsz,ysz])
      IF npix lt 600 THEN $
         scale = 25.0 $
      ELSE IF npix lt 1064 THEN $
         scale = 56.0 $
      ELSE IF npix lt 2100 THEN $
         scale = 112.0 $
      ELSE $
         scale = float(npix)/15.0
      hardim,fim1,0,255,title=imfile1, $
         autosize=1,queue='office',/negative,width=npix/scale
   ENDIF

end
