;+
; NAME:
;	goodpoly
; PURPOSE: (one line)
;	Robust fitting of a polynomial to data.
; DESCRIPTION:
;	This is a multi-pass fitting routine that fits a fixed order polynomial
;	to the input data.  After each pass, the scatter of the fit relative
;	to the fitted line is computed.  Each point is examined to see if it
;	falls beyond THRESH sigma from the line.  If is does, it is removed
;	from the data and the fit is tried again.  This will make two attempts
;	to remove bad data.
; CATEGORY:
;	Function fitting
; CALLING SEQUENCE:
;	coeff = goodpoly(x,y,order,thresh,yfit,newx,newy)
; INPUTS:
;	x      - Input dataset, independant values.
;	y      - Input dataset, dependant values.
;	order  - Order of the polynomial fit (linear = 1).
;	thresh - Sigma threshold for removing outliers.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD PARAMETERS:
;  EXCLUDE - Number of points to exclude from initial pass fit.  This number
;              is rounded up to the next even number.  Then EXCLUDE/2 of the
;              highest and lowest points are removed before the first fit.
;              If these numbers are reasonable, they will not be excluded
;              in the second pass.  This helps prevent biasing the first fit
;              with the worst points in the array.  Default is to not exclude
;              any points on the first pass.
; OUTPUTS:
;	yfit   - Fitted values for y that match the input vector.
;	newx   - X values from input that were considered good.
;	newy   - Y values from input that were considered good.
;	Return value is the set of polynomial coefficients.
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;	Written 1991 Feb., Marc W. Buie, Lowell Observatory
;  93/11/12, MWB, Program fixed to return a computed y for all input x.
;  95/09/20, MWB, Added EXCLUDE keyword.
;-
function goodpoly,x,y,order,thresh,yfit,newx,newy,EXCLUDE=exclude

if badpar(exclude,[0,1,2,3],0,CALLER='goodpoly (EXCLUDE) ',DEFAULT=0) then return,0

arlen=n_elements(x)
xx = x
yy = y

; Initial fit with all the data.
if exclude eq 0 or n_elements(xx) le (exclude+order) then begin
	coeff=poly_fit(xx,yy,order,yfit)

; Initial fit excluding EXCLUDE extrema points before fit.
endif else begin
   nex = ceil(exclude/2.0)
   idx=indgen(n_elements(xx))
   sidx=sort(yy)
   sidx=sidx(nex:n_elements(xx)-1-nex)
	coeff=poly_fit(xx(sidx),yy(sidx),order)
   yfit = poly(xx,coeff)
endelse

chisq0=total((yy-yfit)^2)/(arlen-order)  
flat = (yy-yfit)+(total(yfit)/arlen)
sigma = stdev(flat,mean)
 
;Remove all points beyond threshold sigma
good=where( abs(flat-mean) lt thresh*sigma,goodnum)
nbad = arlen-goodnum 
xx=xx(good)
yy=yy(good)
arlen=n_elements(xx)
 
; Second pass fit with bad points removed.
coeff=poly_fit(xx,yy,order,yfit)
chisq1=total((yy-yfit)^2)/(arlen-order)  
flat = (yy-yfit)+(total(yfit)/arlen) 
sigma = stdev(flat,mean) 
 
;Remove all points beyond threshold sigma
good=where( abs(flat-mean) lt thresh*sigma,goodnum)
nbad = arlen-goodnum
xx=xx(good)
yy=yy(good)
arlen=n_elements(xx)

; Third pass fit with bad points removed.
coeff=poly_fit(xx,yy,order,yfit)
chisq2=total((yy-yfit)^2)/(arlen-order)
flat = (yy-yfit)+(total(yfit)/arlen)
sigma = stdev(flat,mean)

newx=xx
newy=yy

yfit = poly(x,coeff)

return,coeff
end
