;+
; NAME: 
;  ldcalib
; PURPOSE: 
;  Load calibration frames and information as instructed by calib file.
; DESCRIPTION:
; CATEGORY:
;  CCD data processing
; CALLING SEQUENCE:
;  ldcalib,calibfile,calib,valid
; INPUTS:
;  calibfile - Name of calibration file to read.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
;  calib - anonymous structure with calibration information
;           tag contents and usage:
;              xl     - overscan left boundary (-1 means no overscan).
;              xr     - overscan right boundary (-1 means no overscan).
;              x1     - LLHC (x) of cropping region.
;              x2     - URHC (x) of cropping region.
;              y1     - LLHC (y) of cropping region.
;              y2     - URHC (y) of cropping region.
;              bias   - Bias image.
;              dark   - dark image (scalar 0 if no dark correction needed).
;              flat   - Array of flat images.
;              filter - String array of filter names for flat array.
;  valid - Flag, if true indicates a valid calibration set was loaded.  The
;            contents of calib are unreliable if this flag is false.
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  95/08/08, Written by Marc W. Buie, Lowell Observatory
;-
pro ldcalib,calibfile,calib,valid,CALIBPATH=calibpath

valid=0

if badpar(calibfile,7,0,caller='LDCALIB: (calibfile) ') then return
if badpar(calibpath,[0,7],0,caller='LDCALIB: (path) ',default='./') then return

if not exists(addslash(calibpath)+calibfile) then begin
   print,'Calibration file ',addslash(calibpath)+calibfile,' does not exist.  Cannot continue.'
   return
endif

; Pre-declare strings for later use.
version=''
instrument=''
flatinfo=''
biasname=''
darkname=''
filter0=''
flat0=''
xl = -1
xr = -1
x1 = -1
x2 = -1
y1 = -1
y2 = -1

print,'Load calibration info'
openr, lun, addslash(calibpath)+calibfile, /get_lun

;First, read in the calibration information and make sure it is a recognized
;   file version.
readf, lun, version
if version ne 'calib_file_v01' and $
   version ne 'calib_file_v02' and $
   version ne 'calib_file_v03' then begin
   print,'File version tag ['+version+'] is incorrect, cannot proceed.'
   free_lun,lun
   return
endif

if version eq 'calib_file_v01' then begin
   readf, lun, instrument
   if instrument eq 'LCCD' then begin
      readf, lun, biasname
      relpath,biasname,calibpath
      readf, lun, flatinfo
      filter   = gettok(flatinfo,' ')
      flatname = gettok(flatinfo,' ')
      relpath,flatname,calibpath
      while not eof(lun) do begin
         readf, lun, flatinfo
         filter   = [filter,gettok(flatinfo,' ')]
         newflat = gettok(flatinfo,' ')
         relpath,newflat,calibpath
         flatname = [flatname,newflat]
      endwhile
      xl = -1
      xr = -1
      x1 = -1
      x2 = -1
      y1 = -1
      y2 = -1
   endif else if instrument eq 'LORAL' then begin
      readf, lun, xl, xr
      readf, lun, x1, x2, y1, y2
      readf, lun, biasname
      relpath,biasname,calibpath
      readf, lun, flatinfo
      filter   = gettok(flatinfo,' ')
      flatname = gettok(flatinfo,' ')
      relpath,flatname,calibpath
      while not eof(lun) do begin
         readf, lun, flatinfo
         filter   = [filter,gettok(flatinfo,' ')]
         newflat = gettok(flatinfo,' ')
         relpath,newflat,calibpath
         flatname = [flatname,newflat]
      endwhile
   endif else begin
      print,'Unrecognized instrument name [',instrument,'], cannot proceed.'
      free_lun,lun
      return
   endelse
   darkname = '[none]'
endif

if version eq 'calib_file_v02' then begin
   readf, lun, xl, xr, format='(2i)'
   readf, lun, x1, x2, y1, y2
   readf, lun, biasname
   relpath,biasname,calibpath
   readf, lun, flatinfo
   filter   = gettok(flatinfo,' ')
   flatname = gettok(flatinfo,' ')
   relpath,flatname,calibpath
   while not eof(lun) do begin
      readf, lun, flatinfo
      filter   = [filter,gettok(flatinfo,' ')]
      newflat = gettok(flatinfo,' ')
      relpath,newflat,calibpath
      flatname = [flatname,newflat]
   endwhile
   darkname = '[none]'
endif

if version eq 'calib_file_v03' then begin
   readf, lun, xl, xr
   readf, lun, x1, x2, y1, y2
   readf, lun, biasname
   relpath,biasname,calibpath
   readf, lun, darkname
   relpath,darkname,calibpath
   readf, lun, flatinfo
   filter   = gettok(flatinfo,' ')
   flatname = gettok(flatinfo,' ')
   relpath,flatname,calibpath
   while not eof(lun) do begin
      readf, lun, flatinfo
      filter   = [filter,gettok(flatinfo,' ')]
      newflat = gettok(flatinfo,' ')
      relpath,newflat,calibpath
      flatname = [flatname,newflat]
   endwhile
endif

free_lun,lun

print,'Load calibration frames'

;Load the calibration frames.
if not exists(biasname) then begin
   print,'Bias frame ',biasname,' could not be found, aborting.'
   return
endif
bias=readfits(biasname,/silent)
print,'Loading bias  -- ',biasname
sz=size(bias)

if darkname ne '[none]' then begin
   if not exists(darkname) then begin
      print,'Dark frame ',darkname,' could not be found, abortin.'
      return
   endif
   dark=readfits(darkname,/silent)
   print,'Loading dark  -- ',darkname
   nodark=0
endif else begin
   dark = 0
   nodark=1
   print,'Dark calibration step disabled.'
endelse

calib = { $
            xl: xl, $
            xr: xr, $
            x1: x1, $
            x2: x2, $
            y1: y1, $
            y2: y2, $
            bias: bias, $
            dark: dark, $
            flat: fltarr(n_elements(flatname),sz(1),sz(2),/nozero), $
            filter: filter $
        }

for i=0,n_elements(flatname)-1 do begin
   print,'Loading flat (',filter(i),') -- ',flatname(i)
   if not exists(flatname(i)) then begin
      print,'Flat frame ',flatname(i),' could not be found, aborting.'
      return
   endif
   calib.flat(i,*,*) = readfits(flatname(i),/silent)
endfor

if calib.x1 eq -1 then calib.x1 = 0
if calib.x2 eq -1 then calib.x2 = sz(1)-1
if calib.y1 eq -1 then calib.y1 = 0
if calib.y2 eq -1 then calib.y2 = sz(2)-1

valid=1

end
