;+
; NAME: 
;  optspec
; PURPOSE:
;  Optimal extraction of a point source spectrum from OSIRIS XD data.
; DESCRIPTION:
; CATEGORY:
;  Spectroscopy
; CALLING SEQUENCE:
;  optspec,calib,root,i1,i2,spec,all,
;     RAW=raw,NONEG=noneg,AUTO=auto,MASK=mask
; INPUTS:
;  calib- Anonymous structure containing all pertinent calibration
;           information.  This structure is usually loaded beforehand using
;           the routine, "ldcalir"
;  root - string containing the root of the file name (with leading path
;         if desired).  DO NOT include the . between the root and suffix.
;  i1   - Frame id (integer) of first image.
;  i2   - Frame id (integer) of second image.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  AUTO - Flag, if true, enables automatic aperture location for extracting
;         the spectra.   The image is collapsed into a single column.  Any
;         pixel more than 3 sigma from the background is flagged for
;         extraction.  Default is to require clicking on the image displayed
;         in window 0.  Three rows centered on the clicked location are
;         extracted for the spetrum.  /AUTO is the preferred option.
;  APLOC- Override to specify the positive object location.  Overrides AUTO.
;  APSIZE-Override to force the size of the extraction aperture.  The
;           will extend from the location (auto or APLOC) + and - by APSIZE.
;           Thus the total aperture is 2*APSIZE+1 rows.
;  NONEG - Flag, if true, inhibits using the "negative" spectrum in the
;          background image (i2).  Default is to extact the negative
;          spectrum, negate it, and add to the positive spectrum.
;  RAW  - Flag, if true will inhibit the column-wise background subtraction.
;           The default is to fit the background in each column (along the
;           slit) and subtract the fitted background.  Use a linear function
;           for the background.
;  MASK - Indicies in strip image.  These pixels will be replaced by the
;           average of their nearest neighbors in the x direction.
;  SAVE - Flag, if true, final spectrum will be saved to a file.  The output
;           file name is root+'s'+suffix.  Thus 950911.003 would be saved to
;           the file 950911s.003
;  PATH - optional string that points to the directory containing the data.
;           This information is not used if the root already begins with '/'.
;           If root is not an absolute pathname, then PATH is prepended to
;           root for READ operations.  This path is not used for saving.
;           This allows reading from one directory (possible a read only area)
;           and then saving to the current directory.
;  CLEAN - Flag, if set, calls clnspec to allow removing bad points before
;           saving spectrum.
;
; OUTPUTS:
;  spec - 1-D spectrum extracted from spectral image.
;  all  - i1-i2 after XD spectra extracted to strip image. (see getpair)
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
;  Specifically written for OSIRIS cross-dispersed spectral data.  MASK
;  does not work particularly well.
; PROCEDURE:
; MODIFICATION HISTORY:
;  97/03, Written by Marc W. Buie, Lowell Observatory, cloned from getspec
;-

; This is the gaussian with the fixed location (val1=position)
PRO optspec_f4,x,a,f,pder
   common optspec_common,val1,val2,val3,val4,val5
   dx = x - val1
   dx2= dx^2
   e  = exp( -dx2/a(1)^2 )
   f  = a(0) * e
   IF n_params() ge 4 THEN $
      pder = [ [e], [2.0*dx2/a(1)^3*f] ]
END


; This is the gaussian with fixed width and position
PRO optspec_f3,x,a1,a2,f
   dx = x - a1
   dx2= dx^2
   f  = exp( -dx2/a2^2 )
END

; This is the gaussian with fixed width (val1=width)
PRO optspec_f2,x,a,f,pder
   common optspec_common,val1,val2,val3,val4,val5
   dx = x - a(1)
   dx2= dx^2
   e  = exp( -dx2/val1^2 )
   f  = a(0) * e
   IF n_params() ge 4 THEN $
      pder = [ [e], [2.0*dx/val1^2*f] ]
END

; This is the full gaussian function
; f = a(0) exp( -(x-a(1))^2/a(2)^2 )
PRO optspec_f1,x,a,f,pder
   dx = x - a(1)
   dx2= dx^2
   e  = exp( -dx2/a(2)^2 )
   f  = a(0) * e
   IF n_params() ge 4 THEN $
      pder = [ [e], [2.0*dx/a(2)^2*f], [2.0*dx2/a(2)^3*f] ]
END

pro optspec,calib,root,i1,i2,spec,all, $
   RAW=raw,NONEG=noneg,AUTO=auto,MASK=mask,SAVE=save,PATH=path,CLEAN=clean, $
   APSIZE=apsize,APLOC=aploc

   common optspec_common,val1,val2,val3,val4,val5

   if badpar(calib,8,1,CALLER='optspec (calib) ') then return
   if badpar(root,7,0,CALLER='optspec (root) ') then return
   if badpar(i1,[2,3],0,CALLER='optspec (i1) ') then return
   if badpar(i2,[2,3],0,CALLER='optspec (i2) ') then return
   if badpar(raw,[0,1,2,3],0,CALLER='optspec (RAW) ',DEFAULT=0) then return
   if badpar(save,[0,1,2,3],0,CALLER='optspec (SAVE) ',DEFAULT=0) then return
   if badpar(apsize,[0,1,2,3],0,CALLER='optspec (APSIZE) ',DEFAULT=0) then return
   if badpar(aploc,[0,1,2,3],0,CALLER='optspec (APLOC) ',DEFAULT=-1) then return
   if badpar(clean,[0,1,2,3],0,CALLER='optspec (CLEAN) ',DEFAULT=0) then return
   if badpar(auto,[0,1,2,3],0,CALLER='optspec (AUTO) ',DEFAULT=0) then return
   if badpar(mask,[0,2,3],[0,1],CALLER='optspec (MASK) ',DEFAULT=-1) then return
   if badpar(path,[0,7],0,caller='optspec: (path) ',default='./') then return

   if aploc ne -1 then auto=0

   loadct,5,/silent

   if strmid(root,0,1) eq '/' then begin
      newroot = root
   endif else begin
      newroot = addslash(path)+root
   endelse

; Load two images and display
   print,'positive image ',i1,'   negative image ',i2
   getpair,calib,newroot,i1,i2,all,hdr,raw=raw,mask=mask
   all = all * 10.0 ; convert to electrons from counts
   sz=size(all)
   nrows=sz(2)
   ncols=sz(1)
   setwin,0,/show,xsize=ncols,ysize=nrows*4
   erase
   tvscl,all,0

; show image, scaled on sky
   robomean,all,3,.5,avg,dummy,stddev
   tv,bytscl(all,min=avg-3.0*stddev,max=avg+3.0*stddev,top=127),1

   ; Select some random columns to get a first approximation of the
   ;   average column profile.  This will locate the positive and negative
   ;   images.
   nrand=60
   idx=randomu(seed,nrand)*(ncols-1)
   
   avgcol=fltarr(nrows)
   cut=all(idx,*)
   for i=0,nrand-1 do begin
      cut(i,*)=cut(i,*)/max(cut(i,*))
   endfor
   for i=0,calib.height-1 do begin
      robomean,cut(*,i),3,.5,avg
      avgcol(i)=avg
   endfor
   avgcol0=avgcol

   setwin,2,/show
   plot,avgcol

   ; Find the positive and negative peaks
   peakrow=where(avgcol eq max(avgcol))
   peakrow=peakrow(0)
   minrow=where(avgcol eq min(avgcol))
   minrow=minrow(0)

   cut=all
   for i=0,ncols-1 do begin
      cut(i,*)=cut(i,*)/max(cut(i,peakrow-1:peakrow+1))
   endfor
   for i=0,calib.height-1 do begin
      robomean,cut(*,i),3,.5,avg
      avgcol(i)=avg
   endfor

   colno=findgen(calib.height)
   coeff=goodpoly(colno,avgcol,1,2.2,f_col,newx,newy,EXCLUDE=6)
   robomean,newy-poly(newx,coeff),3,.5,bavg,dummy,bstddev

   ;Now we know where the object is so redo background subtraction while
   ;  leaving out the object rows.
   badloc=where( abs(avgcol-f_col-bavg) gt 3.0*bstddev, countbad )
   if countbad ne 0 then begin
      setwin,0
      backsub,all,/col,order=1,exclude=badloc
      tv,bytscl(all,min=avg-3.0*stddev,max=avg+3.0*stddev,top=!d.n_colors-1),2
   endif

   ; Now do the profile again after final sky cleaning
   cut=all
   for i=0,ncols-1 do begin
      cut(i,*)=cut(i,*)/max(cut(i,peakrow-1:peakrow+1))
   endfor
   for i=0,calib.height-1 do begin
      robomean,cut(*,i),3,.5,avg
      avgcol(i)=avg
   endfor

   ; Re-find the positive and negative peaks
   peakrow=where(avgcol eq max(avgcol))
   peakrow=peakrow(0)
   minrow=where(avgcol eq min(avgcol))
   minrow=minrow(0)

   ; From the averaged profile, find the positive definite portion of the
   ;   profile centered on the object.  In other words, scan the profile
   ;   from the peak outward.  The profile should always decrease (or stay
   ;   the same) but never dip below zero.  This will form the template
   ;   for the extraction of the standard spectrum.
   posprofile = avgcol
   FOR i=peakrow+1,calib.height-1 DO BEGIN
      IF posprofile(i) gt posprofile(i-1) or posprofile(i) lt 0.0 THEN $
         posprofile(i)=0.0
   ENDFOR
   FOR i=peakrow-1,0,-1 DO BEGIN
      IF posprofile(i) gt posprofile(i+1) or posprofile(i) lt 0.0 THEN $
         posprofile(i)=0.0
   ENDFOR
   zpos=where(posprofile gt 0.0,numpos)

   ; Do the same for the negative image, just to find out where it is.
   negprofile = avgcol
   FOR i=minrow+1,calib.height-1 DO BEGIN
      IF negprofile(i) lt negprofile(i-1) or negprofile(i) gt 0.0 THEN $
         negprofile(i)=0.0
   ENDFOR
   FOR i=minrow-1,0,-1 DO BEGIN
      IF negprofile(i) lt negprofile(i+1) or negprofile(i) gt 0.0 THEN $
         negprofile(i)=0.0
   ENDFOR
   zneg=where(negprofile lt 0.0,numneg)

   zsky=where(negprofile eq 0.0 and posprofile eq 0.0)

   ; plot profile
   setwin,2,/show
   coeff=goodpoly(colno,avgcol,1,2.2,f_col,newx,newy,EXCLUDE=6)
   robomean,newy-poly(newx,coeff),3,.5,bavg,dummy,bstddev
   plot,colno,avgcol
   oplot,colno,f_col+bavg+3.0*bstddev,color=55
   oplot,colno,f_col+bavg-3.0*bstddev,color=55
   oplot,colno(zpos),posprofile(zpos),color=70,psym=2,symsize=1.5
   oplot,colno(zneg),negprofile(zneg),color=87,psym=2,symsize=1.5
   oplot,colno(zsky),avgcol(zsky),psym=4,color=63,symsize=1.5

   ; extract standard spectrum
   spec = fltarr(ncols)
   FOR i=0,numpos-1 DO $
      spec=spec+all(*,zpos(i))
   setwin,1,/show
   plotspec,calib,spec,yr=[0,6e4]

print,'first fit, all floating'
   ; fit a gaussian to each column over the positive object, all three parameters
   ;   are fitted for this pass.
   a0=fltarr(ncols)
   a1=fltarr(ncols)
   a2=fltarr(ncols)
   wt=replicate(1.0,numpos)
   FOR i=0,ncols-1 DO BEGIN
      acof=[all(i,peakrow),float(peakrow),1.0]
      yfit=curvefit(colno(zpos),all(i,zpos),wt,acof,function_name='optspec_f1')
      a0(i)=acof(0)
      a1(i)=acof(1)
      a2(i)=abs(acof(2))
   ENDFOR

   ; Construct original chip positions.
   xpos=fltarr(ncols)
   ybot=fltarr(ncols)
   FOR i=0,calib.nor-1 DO BEGIN
      xpos(calib.o(i,0):calib.o(i,1))= $
         findgen(calib.o(i,1)-calib.o(i,0)+1)+calib.x1(i,0)
      ybot(calib.o(i,0):calib.o(i,1))= $
         fix(xpos(calib.o(i,0):calib.o(i,1))*calib.slope(i) + calib.y0(i) + 0.5)
   ENDFOR
   yloc=a1+ybot
;   setwin,12
;   plot,xpos,yloc,xr=[0,255],yr=[0,255],psym=4

   ; Given the locations, fit a 2nd order polynomial to the y location
   yfit=fltarr(ncols)
;   !p.multi=[0,1,3]
   FOR i=0,calib.nor-1 DO BEGIN
      coeff=goodpoly(xpos(calib.o(i,0):calib.o(i,1)), $
                     yloc(calib.o(i,0):calib.o(i,1)), $
                     2,2.2,yfit0,newx,newy)
      yfit(calib.o(i,0):calib.o(i,1))=yfit0
;      plot,xpos(calib.o(i,0):calib.o(i,1)), $
;           yfit(calib.o(i,0):calib.o(i,1))- $
;           ybot(calib.o(i,0):calib.o(i,1)),psym=-4
   ENDFOR
   ypos=yfit-ybot
;   !p.multi=0

print,'refit, forcing location'
setwin,13
   ; Refit data, forcing location
   FOR i=0,ncols-1 DO BEGIN
plot,colno,all(i,*)
xyouts,0.75,0.75,string(i),/normal
empty
      val1=ypos(i)
      acof=[max(all(i,zpos)),0.91]
      yfit=curvefit(colno(zpos),all(i,zpos),wt,acof,function_name='optspec_f4')
      a0(i)=acof(0)*acof(1)
      a2(i)=acof(1)
oplot,colno(zpos),yfit,color=55
empty
   ENDFOR

   ; Find mean gaussian width for each order.
   a2mean=fltarr(calib.nor)
   FOR i=0,calib.nor-1 DO BEGIN
      setwin,5+i
      robomean,a2(calib.o(i,0):calib.o(i,1)),3,.5,sigmean,dummy,sigsig
      plot,a2(calib.o(i,0):calib.o(i,1)),yr=[0,sigmean+4*sigsig]
      oplot,calib.o(i,*)-calib.o(i,0),replicate(sigmean,2),color=55
      a2mean(i)=sigmean
      print,i,sigmean,sigsig
   ENDFOR

   ; Add up optimal spectrum, create profile function at the same time.
   optspec=fltarr(ncols)
   prof = fltarr(ncols,numpos)
   FOR i=0,calib.nor-1 DO BEGIN
      FOR j=calib.o(i,0),calib.o(i,1) DO BEGIN
         optspec_f3,colno(zpos),ypos(j),a2mean(i),xfit
         xfit=xfit/total(xfit)
         prof(j,*)=xfit
         optspec(j)=total(all(j,zpos)*xfit)/total(xfit^2)
      ENDFOR
   ENDFOR
   setwin,10
   plotspec,calib,optspec,yr=[0,6e4]

;   setwin,13
;   ans=''
;   FOR i=0,ncols-1 DO BEGIN
;      plot,colno,all(i,*)
;      oplot,colno(zpos),prof(i,*)*optspec(i),color=55
;      xyouts,0.75,0.75,string(i),/normal
;      read,ans,format='(a)'
;   ENDFOR

   ; From spectrum and profile, create "model" image, first create a median
   ;    filtered version of the spectrum
;   smospec=fltarr(ncols)
;   FOR i=0,calib.nor-1 DO BEGIN
;      smospec(calib.o(i,0):calib.o(i,1)) = $
;           median(optspec(calib.o(i,0):calib.o(i,1)),5)
;   ENDFOR
;   setwin,12
;   plotspec,calib,smospec,yr=[0,6e4]

;   model = prof
;   FOR i=0,numpos-1 DO $
;      model(*,i) = model(*,i)*smospec

   ; Construct the difference image between the "model" and the data
;   diff = all(*,zpos)-model

   ; Construct mask image and flag outliers
;   mask=replicate(1.0,ncols,numpos)
;   FOR i=0,calib.nor-1 DO BEGIN
;      robomean,diff(calib.o(i,0):calib.o(i,1),*),3,.5,diffavg,dummy,diffsig
;      zbad = where(abs(diff(calib.o(i,0):calib.o(i,1),*)-diffavg) gt 5.0*diffsig,badcount)
;      print,i,diffavg,diffsig,badcount
;      IF badcount ne 0 THEN BEGIN
;         tmpmask = mask(calib.o(i,0):calib.o(i,1),*)
;         tmpmask(zbad)=0.0
;         mask(calib.o(i,0):calib.o(i,1),*) = tmpmask
;      ENDIF
;   ENDFOR

;   FOR i=0,calib.nor-1 DO BEGIN
;      FOR j=calib.o(i,0),calib.o(i,1) DO BEGIN
;         optspec(j)=total(mask(j,*)*all(j,zpos)*prof(j,*))/ $
;                        total((mask(j,*)*prof(j,*))^2)
;      ENDFOR
;   ENDFOR

;   setwin,11
;   plotspec,calib,optspec,yr=[0,6e4]

;   robomean,spec/optspec,3.0,0.5,scalefac
;   print,scalefac
   spec=optspec

; Clean spectrum if requested.
   if clean then clnspec,calib,spec

; Save spectrum if requested.
   if save then begin
      outname = root+'s.'+string(i1,form='(i3.3)')
      sxaddpar,hdr,'HISTORY','Background from frame '+string(i2)
      writefits,outname,spec,hdr
   endif

end
