;+
; NAME: 
;  plotspec
; PURPOSE: 
;  Plot OSIRIS XD spectral data with wavelength scale.
; DESCRIPTION:
; CATEGORY:
;  Spectroscopy
; CALLING SEQUENCE:
;  plotspec,calib,spec,YRANGE=yrange,TITLE=title,SMOOTH=smo_fac
; INPUTS:
;  calib- Anonymous structure containing all pertinent calibration
;           information.  This structure is usually loaded beforehand using
;           the routine, "ldcalir"
;  spec - 1-D spectrum vector.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  YRANGE = (same as PLOT control, default is min to max)
;  TITLE  = (same as PLOT control, default is blank)
;  SMOOTH = smoothing factor for data, default is no smoothing.  Uses
;           the smooth() function on each order individually.
;  NORDER = Order of spectrum to plot (default=plot all orders).
; OUTPUTS:
;  plot to current output device with wavelength scale.  Each order is
;    plotted a disconnected line from the other orders.
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  95/03/24, Written by Marc W. Buie, Lowell Observatory
;  95/09/15, MWB, added calib structure usage.
;  96/05/28, MWB, mods for new calib structure.
;-
pro plotspec,calib,spec,yrange=in_yr,TITLE=title,SMOOTH=smo_fac, $
   OPLOT=oplot,COLOR=color,xrange=in_xr,THICK=thick,YTITLE=ytitle, $
   NORDER=norder

if badpar(calib,8,1,CALLER='plotspec (calib) ') then return
if badpar(spec,[1,2,3,4,5],1,CALLER='plotspec (spec) ') then return
if badpar(smo_fac,[0,1,2,3],0,CALLER='plotspec (SMOOTH) ',DEFAULT=0) then return
if badpar(norder,[0,1,2,3],0,CALLER='plotspec (NORDER) ',DEFAULT=-1) then return
if !d.name eq 'PS' then defcolor=0 else defcolor=!d.n_colors-1
if badpar(color,[0,1,2,3],0,CALLER='plotspec (COLOR) ',DEFAULT=defcolor) then return
if badpar(thick,[0,1,2,3,4,5],0,CALLER='plotspec (THICK) ',DEFAULT=1.0) then return
if badpar(ytitle,[0,7],0,CALLER='plotspec (YTITLE) ', $
            DEFAULT='Arbitrary Flux') then return

if not keyword_set(title) then title=' '

if norder ge calib.nor then begin
   print,'NORDER=',norder,' is invalid.  Ignoring request and plotting all orders.'
   norder=-1
endif

if not keyword_set(in_xr) then begin
   if norder eq -1 then begin
      xr=minmax(calib.w)
   endif else begin
      xr=minmax(calib.w(calib.o(norder,0):calib.o(norder,1)))
   endelse
endif else begin
   xr = in_xr
endelse

if keyword_set(in_yr) then begin
   yr = in_yr
endif else begin
   if smo_fac eq 0 then begin
      if norder eq -1 then begin
         yr=minmax(spec)
      endif else begin
         yr=minmax(spec(calib.o(norder,0):calib.o(norder,1)))
      endelse
   endif else begin
      if norder eq -1 then begin
         for i=0,calib.nor-1 do begin
            p=smooth(spec(calib.o(i,0):calib.o(i,1)),smo_fac)
            if i eq 0 then $
               yr=minmax(p) $
            else $
               yr=minmax([p,yr])
         endfor
      endif else begin
         yr=minmax(smooth(spec(calib.o(norder,0):calib.o(norder,1)),smo_fac))
      endelse
   endelse
endelse

if not keyword_set(oplot) then $
   plot,[0],[1],/nodata,xr=xr,yr=yr,_EXTRA=e, $
      xtit='Wavelength (microns)',ytit=ytitle,title=title

if norder eq -1 then begin

   for i=0,calib.nor-1 do begin
      if smo_fac eq 0 then $
         p=spec(calib.o(i,0):calib.o(i,1)) $
      else $
         p=smooth(spec(calib.o(i,0):calib.o(i,1)),smo_fac)

      oplot,calib.w(calib.o(i,0):calib.o(i,1)),p,color=color,thick=thick
   endfor

endif else begin

   if smo_fac eq 0 then $
      p=spec(calib.o(norder,0):calib.o(norder,1)) $
   else $
      p=smooth(spec(calib.o(norder,0):calib.o(norder,1)),smo_fac)

   oplot,calib.w(calib.o(norder,0):calib.o(norder,1)),p,color=color,thick=thick

endelse

end
