;+
; NAME:
;    rastr
; PURPOSE: (one line)
;    Convert RA in radians to hours, minutes, and seconds (ASCII string).
; DESCRIPTION:
;
; CATEGORY:
;    Astronomy
; CALLING SEQUENCE:
;    rastr, radians, places, str, carry
; INPUTS:
;    ra     : Right Ascension, in radians, to be converted to a string.
;             May a vector, in which case the outputs will be vectors.
;    places : Resolution of output string (integer).
;             = -7     nearest hour.                    +HH
;             = -6     nearest 30 minutes.              +HH:(00,30)
;             = -5     nearest 15 minutes.              +HH:(00,15,30,45)
;             = -4     nearest 10 minutes.              +HH:M0
;             = -3     nearest 5 minutes.               +HH:M(0,5)
;             = -2     nearest minute.                  +HH:MM
;             = -1     nearest ten seconds.             +HH:MM:S0
;             =  0     nearest second.                  +HH:MM:SS
;             =  1     nearest tenth of a second.       +HH:MM:SS.s
;             =  2     nearest hundredth of a second.   +HH:MM:SS.ss
;             =  3     nearest thousandth of a second.  +HH:MM:SS.sss
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;    str   - Output string for the converted right ascension.
;    carry - Flag that indicates ra rolled over after rounding.
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;    Right ascension is reduced to the range [0,24) then converted to a string.
;    There are no imbedded blanks.
;    Calls external routine radtohms.
; MODIFICATION HISTORY:
;  Copyright (C) 1987, by Marc W. Buie
;  Version dated 87/4/21
;    Ported by Doug loucks, Lowell Observatory, July, 1993, from the
; C-Language version written by Marc Buie.
;-
; ------------------------------------------------------------------------------
; Local procedure dadjust
; ------------------------------------------------------------------------------
PRO dadjust, hh, mm, carry
i = WHERE( mm EQ 60, count )
IF count GT 0 THEN BEGIN
   hh(i) = hh(i) + 1
   mm(i) =  0
   j = WHERE( hh(i) EQ 24, count )
   IF count GT 0 THEN BEGIN
      hh(i(j)) = 0
      carry(i(j)) = 1
   ENDIF
ENDIF
END

; ------------------------------------------------------------------------------
; Local procedure madjust
; ------------------------------------------------------------------------------
PRO madjust, hh, mm, ss, carry
i = WHERE( ss GE 60.0, count )
IF count GT 0 THEN BEGIN
   mm(i) = mm(i) + 1
   ss(i) =  0
   j = WHERE( mm(i) EQ 60, count )
   IF count GT 0 THEN BEGIN
      mm(i(j)) = 0
      hh(i(j)) = hh(i(j)) + 1
      k = WHERE( hh(i(j)) EQ 24, count )
      IF count GT 0 THEN BEGIN
         hh(i(j(k))) = 0
         carry(i(j(k))) = 1
      ENDIF
   ENDIF
ENDIF
END

; ------------------------------------------------------------------------------
; Main procedure rastr
; ------------------------------------------------------------------------------
PRO rastr, ra, places, str, carry
IF N_PARAMS() LT 3 THEN BEGIN
   ; Display the calling sequence.
   PRINT, 'rastr, radians, places, str, carry'
   RETURN
ENDIF

; Allow input radians to be integer, long, floating, or double, and scalar
; or vector.
IF badpar( ra, [2,3,4,5], [0,1], CALLER='RASTR ' ) THEN RETURN

; Allow places to be integer or long scalar.
IF badpar( places, [2,3], 0, CALLER='RASTR ' ) THEN RETURN

true  = 1
false = 0

; First, convert to numerical values of hours, minutes, and seconds.
radtohms, ra, hh, mm, ss

; Initialize the carry parameter.  This statement, although not the most
; efficient, automatically sets 'carry' to have the same rank as 'ra'.
carry = FIX( ra ) * 0

; This is the big decision table that generates the requested format.
CASE places OF
   -7 : BEGIN   ; +HH, Nearest hour.
      i = WHERE( FLOAT(mm) + ss/60.0 GE 30.0, count )
      IF count GT 0 THEN hh(i)=hh(i)+1
      j = WHERE( hh(i) EQ 24, count )
      IF count GT 0 THEN BEGIN
         hh(i(j)) = 0
         carry(i(j)) = true
      ENDIF
      str = STRING( hh, FORMAT='(I2.2)' )
   END

   -6 : BEGIN  ; +HH:(00,30), Nearest 30 minutes.
      mm = nint( ( mm + ss / 60.0 ) / 30.0 ) * 30.0
      dadjust, hh, mm, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' )
   END

   -5 : BEGIN  ; +HH:(00,15,30,45), Nearest 15 minutes.
      mm = nint( ( mm + ss / 60.0 ) / 15.0 ) * 15.0
      dadjust, hh, mm, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' )
   END

   -4 : BEGIN  ; +HH:M0, Nearest 10 minutes.
      mm = nint( ( mm + ss / 60.0 ) / 10.0 ) * 10.0
      dadjust, hh, mm, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' )
   END

   -3 : BEGIN   ; +HH:M(0,5), Nearest 5 minutes.
      mm = nint( ( mm + ss / 60.0 ) / 5.0 ) * 5.0
      dadjust, hh, mm, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' )
   END

   -2 : BEGIN    ; +HH:MM, Nearest minute.
      mm = nint( mm + ss / 60.0 )
      dadjust, hh, mm, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' )
   END

   -1 : BEGIN  ; +HH:MM:S0, Nearest 10 seconds.
      ss = nint( ss / 10.0 ) * 10.0
      madjust, hh, mm, ss, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' ) + ':' + $
      STRING( ss, FORMAT='(I2.2)' )
   END

   0 : BEGIN    ; +HH:MM:SS, Nearest second.
      ss = nint( ss )
      madjust, hh, mm, ss, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' ) + ':' + $
      STRING( ss, FORMAT='(I2.2)' )
   END

   1 : BEGIN  ; +HH:MM:SS.s, Nearest tenth of a second.
      ss = nint( ss / 0.1 ) * 0.1
      ; Save the fractional part of the seconds.
      fss = ss - FIX( ss )
      madjust, hh, mm, ss, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' ) + ':' + $
      STRING( ss, FORMAT='(I2.2)' ) + $
      STRMID( STRING( fss, FORMAT='(F3.1)' ), 1, 2 )
   END

   2 : BEGIN  ; +HH:MM:SS.ss, Nearest hundredth of a second.
      ss = nint( ss / 0.01 ) * 0.01
      ; Save the fractional part of the seconds.
      fss = ss - FIX( ss )
      madjust, hh, mm, ss, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' ) + ':' + $
      STRING( ss, FORMAT='(I2.2)' ) + $
      STRMID( STRING( fss, FORMAT='(F4.2)' ), 1, 3 )
   END

   3 : BEGIN  ; +HH:MM:SS.sss, Nearest thousandth of a second.
      ss = nint( ss / 0.001, /LONG ) * 0.001
      ; Save the fractional part of the seconds.
      fss = ss - FIX( ss )
      madjust, hh, mm, ss, carry
      str = STRING( hh, FORMAT='(I2.2)' ) + ':' + $
      STRING( mm, FORMAT='(I2.2)' ) + ':' + $
      STRING( ss, FORMAT='(I2.2)' ) + $
      STRMID( STRING( fss, FORMAT='(F5.3)' ), 1, 4 )
   END

   ELSE : BEGIN
      MESSAGE, 'PLACES parameter must be in the range -7 to +3.', /INFO
      RETURN
   END
ENDCASE
END
