;+
; NAME: 
;  render
; PURPOSE: 
;  Render a rectangular projection map to a sphere.
; DESCRIPTION:
; CATEGORY:
;  Image display
; CALLING SEQUENCE:
;  render,map,radius,scale,pole,selat,selon,sslat,sslon,nx,ny,image
; INPUTS:
;  map - Array containing a full map of surface
;  radius - Radius of object (eg., in km)
;  scale  - Scale of image (eg., km/pixel)
;  pole   - Position angle of pole, east from north (degrees)
;  selat  - Sub-earth latitude (degrees)
;  selon  - Sub-earth longitude (degrees)
;  selat  - Sub-solar latitude (degrees)  NOT IMPLEMENTED YET
;  selon  - Sub-solar longitude (degrees)  NOT IMPLEMENTED YET
;  nx     - X size of output image in pixels.
;  ny     - Y xize of output image in pixels.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  SILENT - Flag, if set suppresses all printed output
;  NODISPLAY - Flag, if set suppresses graphical output
;
;  Limb darkening model to use (set one of these, at most):
;     
;     LIN   - Linear limb-darkening coeff, map is normal albedo
;     MIN   - Minnaert limb-darkening coeff, map is normal albedo
;     HAPKE - Hapke scattering model, map is single scattern albedo
;                HAPKE(0) = P(0)
;                HAPKE(1) = h (old style, circa 1981)
;  
; OUTPUTS:
;  image  - Rendered image of object.
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;   95/07/24, Written by Marc W. Buie, Lowell Observatory
;   96/04/11, MWB, fixed numerous bugs, now works for sun and viewpoint
;                    being different

;-

pro render,map,radius,scale,in_pole,selat,selon,sslat,sslon,nx,ny,image, $
      LIN=xlimb,MIN=k,HAPKE=hapke,NODISPLAY=nodisplay,SILENT=silent

   ; Save size of input map
   sz=size(map)
   nlon = sz(1)
   nlat = sz(2)

   ; Display input map
   if not keyword_set(nodisplay) then begin
      setwin,1,xsize=nlon,ysize=nlat
      tvscl,map,/order
   endif

   ; Set up output image array
   image=fltarr(nx,ny)

   ; Set up image that contains the y coordinate of each pixel (relative to cntr)
   y = ((findgen(ny) - float(ny)/2.0) * scale)/radius
   onex = replicate(1.0,nx)
   yarr = onex#y

   ; Set up image that contains the x coordinate of each pixel (relative to cntr)
   x = ((findgen(nx) - float(nx)/2.0) * scale)/radius
   oney = replicate(1.0,ny)
   xarr = x#oney

   ; Image of pixel distance from center.
   rsq = xarr^2 + yarr^2

   ; Convert input angles to radians
   pole   = in_pole / !radeg   ; position angle of pole
   theta  = selat   / !radeg   ; sub-earth latitude
   phi    = selon   / !radeg   ; sub-earth longitude
   stheta = sslat   / !radeg   ; sub-solar latitude
   sphi   = sslon   / !radeg   ; sub-solar longitude

   ; Unit vector that points toward observer
   xv = cos(theta)*cos(phi)
   yv = cos(theta)*sin(phi)
   zv = sin(theta)

   ; Unit vector that points toward sun
   xs = cos(stheta)*cos(sphi)
   ys = cos(stheta)*sin(sphi)
   zs = sin(stheta)

   ; Index into IMAGE that fall on disk of object.
   zon = where(rsq le 1.0)

   ; 3-D coordinates of image plane ON the object.  (this is also the set
   ;   of unit vectors for each of the points on the surface of the object)
   x = xarr(zon)
   y = yarr(zon)
   z = sqrt( 1.0 - rsq(zon) )

   ; Rotate the surface vectors to the viewpoint
   rotpoint,x,y,z,'z',-pole,xp,yp,zp
   rotpoint,xp,yp,zp,'x',-theta,xpp,ypp,zpp
   rotpoint,xpp,ypp,zpp,'y',phi,xppp,yppp,zppp

   ; Compute dot products with viewpoint and illumination directions
   mu  = zppp*xv + xppp*yv + yppp*zv
   smu = zppp*xs + xppp*ys + yppp*zs

   ; Compute map coordinates for each surface pixel
   colat = acos(yppp)
   lon   = atan(xppp,zppp)

   ; Convert negative longitudes to positive.
   zl = where(lon lt 0,count)
   if count ne 0 then lon(zl)=lon(zl)+!pi*2.0

   ; Convert from lat,lon to indicies into the map, these arrays now point
   ;   to the visbile pixels, but they are not necessarily illuminated.
   xidx = long(lon/!pi/2.0*nlon+0.5)
   yidx = long(colat/!pi*nlat+0.5)

   ; Find only those pixels that are illuminated
   ill = where(smu gt 0.,ill_count)

   ; compute image pixels for relevant reflectance "law"
   if ill_count gt 0 then begin

      ; Linear limb darkening case (Marcialis)
      if keyword_set(xlimb) then begin
         if not keyword_set(silent) then print,'Linear limb darkening'
         image(zon(ill)) = map(xidx(ill),yidx(ill))*(1.0-xlimb+xlimb*mu(ill))

      ; Minnaert limb darkening
      endif else if keyword_set(k) then begin
         if not keyword_set(silent) then print,'Minnaert limb darkening'
         image(zon(ill)) = map(xidx(ill),yidx(ill))*mu(ill)^(2*k-1.0)

      ; Hapke scattering function.
      endif else if keyword_set(hapke) then begin
         if not keyword_set(silent) then print,'Hapke theory used.'
         image(zon(ill)) = bidr(map(xidx(ill),yidx(ill)),mu(ill),smu(ill),hapke(0),hapke(1))*!pi

      ; No limb darkening at all, just wrap the map as is.
      endif else begin
         if not keyword_set(silent) then print,'No limb darkening used.'
         image(zon(ill)) = map(xidx(ill),yidx(ill))
      endelse

   endif

   ; Display final image.
   if not keyword_set(nodisplay) then begin
      setwin,0,xsize=nx,ysize=ny
      tvscl,image
   endif

end
