;+
; NAME:
;	rephot
; PURPOSE: (one line)
;	Reprocess photometry data set by re-extracting from images.
; DESCRIPTION:
; CATEGORY:
;       Photometry
; CALLING SEQUENCE:
; INPUTS:
;	oldlog - Filename of an alternate format basphot log file.
;	newlog - Filename for the new format file (must be different).
;	calibs - Filename for the list of calibration files to be used.
;
;         Version 1 files:
;
;	           Line 1 - ID line, "calib_file_v01"
;	           Line 2 - Instrument  (rest of file depends on this).
;
;	       LCCD
;	           Line 3 - Name, including path, of bias frame.
;	           Line 4 - Filter code, and file name of flat field
;	                       image for this filter.
;	            additional lines with filter and flats.
;
;              LORAL
;                  Line 3 - overscan limits (x1, x2)
;	           Line 4 - cropping region (x1:x2,y1:y2)
;	           Line 5 - Name, including path, of bias frame.
;	           Line 6 - Filter code, and file name of flat field
;	                       image for this filter.
;	            additional lines with filter and flats.
;
;         Version 2 files:
;             Line 1 - ID line, calib_file_v02
;             Line 2 - overscan limits (x2,x2), if none, use -1 -1
;             Line 3 - cropping region (x1:x2,y1:y2)
;	           Line 4 - Name, including path, of bias frame.
;	           Line 5 - Filter code, and file name of flat field
;	                       image for this filter.
;	            additional lines with filter and flats.
;
;         Version 3 files:
;             Line 1 - ID line, calib_file_v02
;             Line 2 - overscan limits (x2,x2), if none, use -1 -1
;             Line 3 - cropping region (x1:x2,y1:y2)
;	           Line 4 - Name, including path, of bias frame.
;	           Line 5 - Name, including path, of dark frame.  [none] means not needed.
;	           Line 6 - Filter code, and file name of flat field
;	                       image for this filter.
;	            additional lines with filter and flats.
;
; OPTIONAL INPUT PARAMETERS:
; KEYWORD PARAMETERS:
;	PATH    - Directory path pointing to location for raw images.
;	NEWGAIN - New value of the gain to reduce with, e-/ADU
;	NEWRAD  - New value of the object aperture radius in pixels.
;	NEWSKY1 - New value of the inner sky annulus radius in pixels.
;	NEWSKY2 - New value of the outer sky annulus radius in pixels.
;  PSCALE  - Provide plate scale of data (default = 0.726 arcsec/pixel).
;	SILENT  - Do not generate voluminous BASPHOTE output to the screen.
;	NSTART  - Starting point number to measure (default is beginning).
;	NSTOP   - Ending point number to measure (default is end).
;	              Note: NSTART and NSTOP are zero-indexed.
; OUTPUTS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;	Written 1993 Feb 4 by Marc W. Buie, Lowell Observatory
;	93/05/10, Added LORAL instrument option, MWB.
;  93/11/02, MWB, added PSCALE keyword, fixed overscan code for LORAL
;  94/03/16, MWB, added version 2 calibration file support.
;  95/03/03, MWB, fixed bug the didn't handle -1 -1 in calib files properly
;  95/03/07, MWB, added version 3 calibration file support.
;  95/06/12, MWB, added CALIBPATH support.
;  95/08/08, MWB, split out calibration info loading to separate routine.
;  96/10/31, MWB, added "bad" flag support from rdphalt
;-
pro rephot,oldlog,newlog,calibs, $
   CALIBPATH=calibpath, $
   PATH=path,NEWGAIN=newgain,NEWRAD=newrad,NEWSKY1=newsky1,NEWSKY2=newsky2, $
   NSTART=nstart,NSTOP=nstop,PSCALE=pscale,SILENT=silent

;Validate parameters.
if n_params() ne 3 then begin
   print,'rephot,oldlog,newlog,calibs'
   return
endif

if badpar(oldlog,7,0,caller='REPHOT: (oldlog) ') then return
if badpar(newlog,7,0,caller='REPHOT: (newlog) ') then return
if badpar(calibs,7,0,caller='REPHOT: (calibs) ') then return

if badpar(path,[0,7],0,caller='REPHOT: (path) ',default='./') then return
path=addslash(path)
defcalib=path+'calib/'
if badpar(calibpath,[0,7],0,caller='REPHOT: (calibpath) ',default=defcalib) then return
if badpar(newgain,[0,2,3,4,5],0,caller='REPHOT: (newgain) ') then return
if badpar(newrad,[0,2,3,4,5],0,caller='REPHOT: (newrad) ') then return
if badpar(newsky1,[0,2,3,4,5],0,caller='REPHOT: (newsky1) ') then return
if badpar(newsky2,[0,2,3,4,5],0,caller='REPHOT: (newsky2) ') then return
if badpar(pscale,[0,4,5],0,caller='REPHOT: (pscale) ',default=0.726) then return

if not exists(oldlog) then begin
   print,'Old log file ',oldlog,' does not exist.  Cannot continue.'
   return
endif

; Load calibration information
ldcalib,calibs,calib,valid,calibpath=calibpath
if not valid then return

;Second, read in the old log file and get all the information.

print,'Load old photometry from ',oldlog

rdphalt,oldlog,filename,obj,fil,jd,exptime,gain,rad,sky1,sky2, $
            serial,xpos,ypos,fwhm,maxcnt,mag,err,bad

if keyword_set(newgain) then gain(*)=newgain
if keyword_set(newrad)  then rad(*) =newrad
if keyword_set(newsky1) then sky1(*)=newsky1
if keyword_set(newsky2) then sky2(*)=newsky2

nobs = n_elements(obj)

print,'Validate calibration files against photometry logfile.'
;Validate the filter codes and calibrations files before doing photometry.
flatcode=intarr(nobs)
for i=0,nobs-1 do begin
   flatcode0 = where(fil(i) eq calib.filter,count)

   if count eq 0 then begin
      print,'ABORTING: A flat field for filter code [',fil(i),'] was not found.'
      print,'You must fix the photometry file or the calibration file before proceeding.'
      return
   endif

   if count ge 2 then begin
      print,'ABORTING: the calibration file has a multiple flat definition for filter ',fil(i)
      return
   endif

   flatcode(i) = flatcode0

endfor

if badpar(nstart,[0,2,3],0,caller='REPHOT: (nstart) ',default=0) then return
if badpar(nstop,[0,2,3],0,caller='REPHOT: (nstop) ',default=nobs-1) then return

if nstart lt 0 then begin
   print,'WARNING: nstart less than zero.  Reset to start.'
   nstart=0
endif

if nstop ge nobs then begin
   print,'WARNING: nstop larger than number of points in file.  Reset to end.'
   nstop=nobs-1
endif

if nstart gt nstop then begin
   print,'ERROR: nstart is bigger than nstop, unable to continue.'
   return
end

print,'Ok, Start doing photometry, points ',nstart,' to ',nstop
i=nstart
while (i le nstop) do begin

   for j=i+1,nstop do begin
      if filename(j) ne filename(i) then begin
         last=j-1
         goto,found_range
      endif
   endfor

   last=nstop   ; This is the loop fall through condition (end of list).

   found_range:

   print,i,last-i+1,filename(i),obj(i),nstop-i, $
      format='("Point ",i4,1x,i2,"  file ",a,"  object [",a,"]   left:",i4)'

   ;Process image.

   if not exists(path+filename(i)) then begin
      print,'Image file ',path+filename(i),' could not be found.  Aborting.'
      return
   endif
   image = readfits(path+filename(i),/silent)

   ; Overscan correction.
   if calib.xl ge 0 and calib.xr ge 0 then $
      os = mean( image(calib.xl:calib.xr,*) ) $
   else $
      os = 0.0

   if n_elements(calib.dark) eq 1 then $
      image = (image(calib.x1:calib.x2,calib.y1:calib.y2) - calib.bias - os) / $
               calib.flat(flatcode(i),*,*) $
   else $
      image = (image(calib.x1:calib.x2,calib.y1:calib.y2) - calib.bias - $
                                                calib.dark*exptime(i) - os ) / $
                           calib.flat(flatcode(i),*,*)

   ;Do the photometry
   basphote,gain(i),image,exptime(i),xpos(i:last),ypos(i:last),rad(i:last), $
               sky1(i:last),sky2(i:last),newlog,serial(i:last),/altlog, $
               fname=filename(i),filter=fil(i),jd=jd(i),name=obj(i:last), $
               pscale=pscale,silent=silent,bad=bad(i:last)

   i=last+1

endwhile

end
