;+
; NAME: 
;  setwin
; PURPOSE: 
;  Set current draw window, create if needed.
; DESCRIPTION:
;  This is intended to be a more general call for controlling the use of
;  multiple plot windows especially in scripts or programs.  In particular,
;  this routine becomes a no-op if the current display device can't handle
;  windows (like the PS device for printing).  In its simplest form, just
;  type   setwin,number   to make window number the current window.  If the
;  window does not exist it will be created.  The other optional input work
;  as described in the IDL documentation for WDEL, WSHOW, WINDOW, WSET.
;  Note that there is no way I've found to access the current position of
;  a window on a screen so if you specify XPOS or YPOS, then this will always
;  cause the window to be destroyed and re-created.  For XSIZE and YSIZE,
;  the window is not destroyed if the size doesn't change.
; CATEGORY:
;  Utility
; CALLING SEQUENCE:
;  setwin,id
; INPUTS:
;  id - window number to make the current draw window
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  ERASE - Flag, if set causes the window to be erased after setting.
;  SHOW  - Flag, if set causes the window to be brought to the front.
;  XPOS  - X position of the lower left corner of the window
;  YPOS  - Y position of the lower left corner of the window
;  XSIZE - the x size of the window.
;  YSIZE - the y size of the window.
; OUTPUTS:
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  94/04/26 - Written by Marc W. Buie, Lowell Observatory
;-
pro setwin,id, ERASE=erase,SHOW=show, $
               XPOS=xpos,YPOS=ypos,XSIZE=xsize,YSIZE=ysize

   if badpar(id,[1,2,3],0,CALLER='SETWIN: id ') then return

   if id lt 0 or id ge 32 then begin
      print,'SETWIN: window id must be >= 0 and < 32'
      return
   endif

   ; First, check to see if this is a windowing device.
   if (!d.flags and 256) ne 0 then begin

      ; This is a bit of a kludge, explanation at the end.
      create = 0

      ; Next, check to see if window exists
      device,window_state=win_state

      ; If yes, check xsize,ysize,xpos,ypos, if different delete window
      ;   and create it again
      if win_state(id) eq 1 then begin
         wset,id
         if not keyword_set(xsize) then xsize = !d.x_size
         if not keyword_set(ysize) then ysize = !d.y_size
         if xsize ne !d.x_size or ysize ne !d.y_size or $
               keyword_set(xpos) or keyword_set(ypos) then begin
            wdelete,id
            create=1
         endif

      ; If no, create window
      endif else begin
         create=1
      endelse

      ; If create flag set we must create it.  The problem is, not all of
      ;    the keywords may be specified.  If not specified, they can't appear
      ;    on the command line so we must build a string then EXECUTE it.
      ; Please tell me if there is an easier way...
      if create eq 1 then begin
         cmd = 'window,id'
         if keyword_set(xpos) then cmd=cmd+',xpos=xpos'
         if keyword_set(ypos) then cmd=cmd+',ypos=ypos'
         if keyword_set(xsize) then cmd=cmd+',xsize=xsize'
         if keyword_set(ysize) then cmd=cmd+',ysize=ysize'
         r = execute(cmd)
         if r eq 0 then print,'SETWIN: EXECUTE command failed'
      endif

      ; Erase if set
      if keyword_set(erase) then erase

      ; Bring to front if SHOW set
      if keyword_set(show) then wshow

   ; If postscript device, then just adjust xsize and ysize according to
   ;   the input.  The goal is to keep the aspect ratio the same.  This does
   ;   something only if BOTH xsize and ysize are set.
   endif else if !d.name eq 'PS' then begin
      if keyword_set(xsize) and keyword_set(ysize) then begin
         aspect = float(ysize)/float(xsize)
         if aspect lt 1.0 then begin ; more wide than tall
            device,ysize=!d.x_size/!d.x_px_cm*aspect
         endif else begin ; more tall than wide
            device,xsize=!d.y_size/!d.y_px_cm/aspect
         endelse
      endif
   endif

end
