;+
; NAME:
;    sumann
; PURPOSE: (one line)
;    Integrate over an annulus.
; DESCRIPTION:
;
; CATEGORY:
;    CCD data processing
; CALLING SEQUENCE:
;    Sumann, image, xcen, ycen, inradius, outradius, back, totweight, $
;            possum, negsum, posxmom, negxmom, posymom, negymom
;
; INPUTS:
;    image       : CCD image array.
;    xcen,ycen   : Center of annulus.
;    inradius    : Radius of inner circle.
;    outradius   : Radius of outer circle.
;    back        : Background to subtract from each pixel.
;
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;    totweight          : Area of annulus.
;    possum, negsum     : Sums of positive and negative pixels.
;    posxmom, negxmom   : Positive and negative x moments.
;    posymom, negymom   : Positive and negative y moments.
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;    Ported by Doug Loucks, Lowell Observatory, 1992 Oct, from the
;    C-language version written by Marc Buie.
;    April, 1993. DWL. Replaced the inner FOR loop with vectors.
;-
PRO Sumann, image, xcen, ycen, inradius, outradius, back, totweight, $
            possum, negsum, posxmom, negxmom, posymom, negymom

totweight = 0.0

possum  = 0.0
negsum  = 0.0
posxmom = 0.0
negxmom = 0.0
posymom = 0.0
negymom = 0.0

image_size = SIZE( image )

r2 = outradius + 2
npts = LONG( !PI * r2*r2 )
x = INTARR( npts, /NOZERO )
y = INTARR( npts, /NOZERO )
flags = INTARR( npts )

;  As the pixel location vectors (x, y) are assembled, the vector flags
;  is constructed with the following values:
;
;     0  : Trivial.  Pixwt not needed.
;     1  : Trivial.  Pixwt not needed.
;    -1  : On or near outer circle only.
;    -2  : On or near inner circle only.
;    -4  : On or near both circles.
;
;  Then the flags are used to calculate the final weights for each pixel
;  location.
;
IF ( 0 LE inradius ) AND ( inradius LT outradius ) THEN BEGIN
   ; Compute the y-range limits.
   r2 = outradius
   r1 = inradius
   r2sq = r2 * r2
   r1sq = r1 * r1
   yp = ycen - r2
   outy0 = FIX( yp )
   IF outy0+0.5 LT yp THEN outy0=outy0+1
   yp = ycen + r2
   outy3 = FIX( yp + 1 )
   IF outy3-0.5 GT yp THEN outy3=outy3-1

   base = 0
   FOR yp = outy0, outy3 DO BEGIN
      Cgetrng, ycen, xcen, r2, yp, outx0, outx1, outx2, outx3
      Cgetrng, ycen, xcen, r1, yp, inx0, inx1, inx2, inx3
      n = outx3 - outx0 + 1

      ;  Assume on outer circle.
      xx = outx0 + INDGEN( n )
      yy = REPLICATE( yp, n )
      f  = REPLICATE( -1, n )

      ;  Entirely inside inner circle.
      t = WHERE( outx1 LE xx AND xx LT outx2, count )
      IF count GT 0 THEN f(t) = 1

      ;  On inner circle.
      t = WHERE( (inx0 LE xx AND xx LT inx1) OR $
      (inx2 LE xx AND xx LT inx3), count )
      IF count GT 0 THEN f(t)=f(t)-3

      ;  Entirely inside inner circle.
      t = WHERE( inx1 LE xx AND xx LT inx2, count )
      IF count GT 0 THEN f(t)=0
      
      x( base : base+n-1 ) = xx
      y( base : base+n-1 ) = yy
      flags( base : base+n-1 ) = f
      base = base + n
   ENDFOR

   x = x( 0 : base-1 )
   y = y( 0 : base-1 )
   flags = flags( 0 : base-1 )

   t = WHERE( flags NE 0, count )
   IF count GT 0 THEN BEGIN
      x = x(t)
      y = y(t)
      flags = flags(t)
   ENDIF

   ; Select only those pixels which are actually in the image array.
   xmin = MIN( x, MAX=xmax )
   ymin = MIN( y, MAX=ymax )
   IF xmin LT 0 OR xmax GE image_size(1) OR $
      ymin LT 0 OR ymax GE image_size(2) THEN BEGIN
      i = WHERE( x GE 0 AND x LE image_size( 1 ) AND $
                 y GE 0 AND y LE image_size( 2 ), count )
      IF count EQ 0 THEN RETURN
      x = x( i )
      y = y( i )
      flags = flags( i )
   ENDIF

   wts = FLOAT( flags )

   i = WHERE( flags EQ -1, count )
   IF count GT 0 THEN wts(i) = Pixwt( xcen, ycen, outradius, x(i), y(i) )

   i = WHERE( flags EQ -2, count )
   IF count GT 0 THEN wts(i) = 1.0 - Pixwt( xcen, ycen, inradius, x(i), y(i) )

   i = WHERE( flags EQ -4, count )
   IF count GT 0 THEN BEGIN
      wts(i) = Pixwt( xcen, ycen, outradius, x(i), y(i) ) - $
               Pixwt( xcen, ycen, inradius, x(i), y(i) )
   ENDIF

   totweight = TOTAL( wts, /DOUBLE )

   val = image( x, y )
   vxw = ( val - back ) * wts

   i = WHERE( vxw GT 0, count )
   IF count NE 0 THEN BEGIN
      possum  = TOTAL( vxw( i ), /DOUBLE )
      posxmom = TOTAL( vxw( i ) * ( x( i ) - xcen ), /DOUBLE )
      posymom = TOTAL( vxw( i ) * ( y( i ) - ycen ), /DOUBLE )
   ENDIF

   i = WHERE( vxw LT 0, count )
   IF count NE 0 THEN BEGIN
      negsum  = TOTAL( vxw( i ), /DOUBLE )
      negxmom = TOTAL( vxw( i ) * ( x( i ) - xcen ), /DOUBLE )
      negymom = TOTAL( vxw( i ) * ( y( i ) - ycen ), /DOUBLE )
   ENDIF

ENDIF
END
